#include <hb.h>
/*
 * File: cbody.c
 *
 * Purpose: This has the one function that's been modified to be used for
 *          C indexing vs. the Fortran standard of the HB file format.
 *
 * Contents: CProcInts 
 *
 * Modified: 3/12/94 -- clean up comments a little 
 */

/*
 * Function: CProcInts
 *
 * Purpose: Process lines of the file (stored in buffer) using
 *          the format held in format.  These lines are assumed
 *          to contain FORTRAN array indices (1-n) that are to be converted to
 *          the C array standard (0-n-1)
 *
 * Parameters: format = the format to be used
 *             buffer = the input lines
 *             array = the processed integers
 *             lines = the number of lines expected
 *             bufsize = the size in bytes of the buffer
 *
 * Return Value: the number of bytes processed
 *
 * Calls: FormatInt GetBuffLine
 * 
 * Programmer: Tom Loos, tloos@cs.indiana.edu, LH 310, 812 855-3609
 *
 * Started: 2/9/94
 *
 * Modified: 2/12/94 -- clean up comments
 *           10/28/94 -- fix overflow in last line bug 
 */
int CProcInts(Format *format, char *buffer, int *array, int *lines, int bufsize)
{
    char *line;
    int nl_seen;
    int i;
    int bytecount;
    int ints_per_line = format->count;
    int int_size = format->width;
    int cur_byte;
    int cur_ints_per_line; /* 10/28/94 */
    int total = 0;
/* DEBUG ACTION HERE */
/*    FILE *fubar;
 *
 *   if ((fubar=fopen("fubar","w")) == NULL)
 *   {
 *	fubar = stderr;
 *   }
 */

    /* while there's work */
    while (*lines > 0)
    {
	/* get a line from the buffer */
	bytecount = GetBuffLine(buffer, &line, MAX_LINE_SIZE, &nl_seen);

	/* if no newline seen, need to re-read, so break */
	if ((!nl_seen) || ((total+bytecount) > bufsize))
	{
	    break;
	}
	/* else */

	/* 10/28/94 -- see if we can fix overflow problem */
	cur_ints_per_line = bytecount/int_size;
/*
 * DEBUGGING CODE
 *
 * 	fprintf(fubar,"cur_ints_per_line = %d, ints_per_line = %d\n",
 *		cur_ints_per_line, ints_per_line);
 */
	/* for each integer in the current line, format it and go on */
	cur_byte = 0;
/*	for (i = 0; i < ints_per_line; i++) */
	for (i = 0; i < cur_ints_per_line; i++) /* 10/28/94 */
	{
	    *array = FormatInt( &(line[cur_byte]), int_size )-1; /* -1 for c */
	    cur_byte += int_size;
/*  fprintf(fubar,"CProcInts: value = %d\n",array[0]);  */
	    array++; /* move onto next element */
	}
	/* update total */
	total += bytecount;

	/* decrement number of lines   */
	(*lines)--;
	buffer += bytecount;
    }
/* MORE DEBUG ACTION */
/*    if (fubar != stderr)
 *	fclose(fubar);
 */
    return( total );
}


