/*
 * File: emily.inter.c -- interface file to emily for matlab
 *
 * References:
 *  Users' Guide for the Harwell-Boeing Sparse Matrix Collection (Release I),
 *     Duff, Grimes, and Lewis, publisher?, Oct. 1992
 *
 *  Matlab External Interface Guide, The Math Works, Natick, Mass, Aug. 1992
 *
 * Includes: mexFunction for emily()
 *
 * Started: 3/6/94 from hbin.inter.c
 *
 * Modified: 3/6/94 -- clean up comments
 *           8/30/94 -- add calc. input min/max loop for matlab sparse mat.
 *                      input.
 *           9/20/94 -- changed mexFunction() to be a void function
 *
 */

#include <hb.h>

/* globals */
DaMatrix d;
HBMatrix hb;
DenseMatrix dense;

/*
 * Function: mexFunction
 *
 * Purpose: to provide an interface between HB input routines and matlab
 *
 * Usage: emily('matrix_file')
 *
 * Parameters: STRING_IN - name of the input file
 *
 * Return Values: none
 * 
 * Calls: mexErrMsgTxt mxGetM mxGetN mxIsString malloc mxGetPr DrawMat
 *
 * Notes: this is unintelligible code without the Matlab External
 *        Interface Guide.  Also, this code does very little actual
 *        work -- it's all done by either matlab or DrawMat
 *
 * Programmer: Tom Loos, Indiana Univ, LH310, +01 (812) 855-3609,
 *             tloos@cs.indiana.edu
 *
 * Started:   3/6/94
 *
 * Modified:
 *
 */

/* determine matlab MexFunction type depending on -DMATLAB_42 */
#ifdef MATLAB_42
void 
#else
int 
#endif
#ifdef __STDC__
mexFunction(int		nlhs,
	    Matrix	*plhs[],
	    int		nrhs,
	    Matrix	*prhs[]
	    )
#else
mexFunction(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
Matrix *plhs[], *prhs[];
#endif
{
    double	*data;
    int	rows,width,i, code;
    int sparse;
    char s[200];
    int nptns, ptn_vect[1000];
    int d1;
    int nzmax = 0;
    int *col_indices = (int *) NULL;
    int *row_indices = (int *) NULL;
    struct res r;
    HBMatrix hb;
    DenseMatrix den;
    double absval; /* 8/30/94 */

    /* Check for proper number of arguments */

    if ( (!((nrhs == 1) || (nrhs == 2))) || (nlhs != 0)) 
    {
	mexErrMsgTxt("usage: emily(M), where M is a matrix");
    }
    /* 7/29/96 handle 2nd RHS argument */
    if (nrhs == 2)
    {
	if (!mxIsString(BSTART_IN))
	    mexErrMsgTxt("emily's second argument must be the name of a partition vector.");
	/* get string data */
	data = mxGetPr(BSTART_IN);
	width = mxGetN(BSTART_IN);
	for (i = 0; i < width; i++)
	{
	    d1=(int) data[i];
	    s[i] = (char) d1;
	}
	s[width] = '\0';
	ReadPtnFile( s, ptn_vect, &nptns );
	/* printf("read in bstart file %s. nptns = %d\n",s,nptns); */
    }
    else
	nptns = 0;

    /* get row and column info */
    rows = mxGetM(STRING_IN);
    width = mxGetN(STRING_IN);

    /* check for input string */
    if (!(mxIsString(STRING_IN) || mxIsFull(STRING_IN) || mxIsSparse(STRING_IN)) && (rows != 1))
    {
	mexErrMsgTxt("usage: emily(M), where M is a matrix with 1 row");
    }
    
    /* if input is a string, treat as a file */
    if (mxIsString(STRING_IN))
    {
	data = mxGetPr(STRING_IN);
	code = DRAW_MAT_USE_FILENAME;
	/* transform input data to a string */
	/* i'm pretty sure there's a matlab function for this, but oh, well */
	for (i = 0; i < width; i++)
	{
	    d1 = (int) data[i];
	    s[i] = (char) d1;
	}
	s[width] = '\0';
    }
    else /* it's a bona-fide matrix */
    {
	code = DRAW_MAT_USE_MATRIX;

	if (mxIsSparse(STRING_IN)) /* matrix is sparse */
	{
	    /* set up HB header */
	    d.type = HB;
	    d.data.hb = &hb;
	    /* fill out what we can of the header */
	    hb.head.NumRows = mxGetM(STRING_IN);
	    hb.head.NumCols = mxGetN(STRING_IN);
	    hb.head.NumRowIndex = mxGetNzmax(STRING_IN);

	    /* finish up the rest */
	    hb.head.RHSLines = 0;
	    hb.head.NumRHS = 0;
	    hb.head.NumericalData = IsReal;
	    hb.head.Symmetry = IsUnsymmetric; /* by default, i guess */
	    hb.head.Assembly = IsAssembled;
	    hb.head.NumElemMatEntries = 0;

	    strcpy(hb.head.Title,"1Input From Matlab");
	    strcpy(hb.head.Key,"Matlab1");
	    hb.head.RHSLines = 0;
	    
	    /* get the data */
	    hb.values = mxGetPr(STRING_IN);
	    /* 8/30/94 -- set min and max values */
	    if (hb.head.NumRowIndex > 0)
	    {
		/* init min, max, etc. values */
		hb.head.MaxValue = hb.head.MinValue = hb.values[0];
		hb.head.AbsMaxValue = myabs((hb.values[0]));
		hb.head.AbsMinValue = myabs((hb.values[0]));
		/* loop thru the rest of the values looking for the 
		   extrema (start with 1 on purpose) */
		for (i = 1; i < hb.head.NumRowIndex; i++)
		{
		    /* assign max/min */
		    if (hb.values[i] > hb.head.MaxValue)
			hb.head.MaxValue = hb.values[i];

		    if (hb.values[i] < hb.head.MinValue)
			hb.head.MinValue = hb.values[i];

		    /* now for abs values */
		    absval = myabs((hb.values[i]));
		    if (absval > hb.head.AbsMaxValue)
			hb.head.MaxValue = absval;

		    if (absval < hb.head.AbsMinValue)
			hb.head.AbsMinValue = absval;
		} /* end of for i loop */
	    } /* end of if statement */
/*
 * fprintf(stderr,"inter.c: max = %e, min = %d, absmax = %e, absmin = %e\n",
 *	hb.head.MaxValue, hb.head.MinValue,
 *	hb.head.AbsMaxValue,hb.head.AbsMinValue);
 */	
	    /* get col indices */
	    hb.col_inds = mxGetJc(STRING_IN);

	    /* get row indices */
	    hb.row_inds = mxGetIr(STRING_IN);

	    /* finish up the matrix */
	    hb.RHS = (double *) NULL;
	    hb.guess = (double *) NULL;
	    hb.exact = (double *) NULL;
	}
	else /* Dense */
	{
	    d.type = Dense;
	    d.data.d = &den;
	    /* get M and N from input matrix */
	    den.NumRows = mxGetM(STRING_IN);
	    den.NumCols = mxGetN(STRING_IN);

	    /* set up storage requirements */
	    /* matlab stores dense/full mats. in col major order
	       in a 1 D array */
	    den.row_major = 0;
	    den.one_d = 1;

	    /* get input data */
	    den.data.d1 = mxGetPr(STRING_IN);
	}
 	/* hack to pass an address as a string */
 	/* sprintf(s,"%d",&d); */
	/* 1/30/95 -- don't pass address as string anymore */
	strcpy(s,"test_matrix\n");
	/* fprintf(stderr,"inter.c: input_mat = %s, addr = %d\n",s,&d); */
   }
    BigDrawMat2(s,code,&d, nptns, ptn_vect);
    /* fprintf(stderr,"calling BDM!\n"); */
}


