/*
 * hb.h -- header file for HB sparse matrix input/output
 * 
 * HB (Harwell Boeing) format from "User's Guide for the Harwell-Boeing
 *    Sparse Matrix Collection (Release 1)", Duff, Grimes, and Lewis,
 *    October 1992.
 *
 * Started: 1/29/94 by Tom Loos (tloos@cs.indiana.edu)
 * 
 * Modified: 2/10/94  -- changed it to just include hbmat.h
 *           2/13/94  -- moved all includes here and added includes
 *                      of hbdraw.h and X header files
 *           3/1/94   -- added unistd.h
 *           3/18/94  -- added debugging stuff (DEBUG_LEVEL and TRACE_ON)
 */

/* includes */
#ifndef HB_H
#define HB_H

/* global consts that all files will use */
#define EPSILON 1e-15
#define BITMAP_EPSILON 1e-150
#define SCALE_EPSILON 1e-300

/* debugging level 0 silent -> 3 max output */
#ifndef DEBUG_LEVEL
#define DEBUG_LEVEL 0
#endif 

/* DISPLAY_TIMING: instrumentation: display read, compress, etc. times flag */
#define DISPLAY_TIMING 0 
/* function entry/exits printed if TRACE_ON is on */
#define TRACE_ON 0

/* sys includes */
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h> /* for LSEEK */
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h> 
/* for stat */
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
/* for mytime ==> getrusage() */
#include <sys/time.h>
#include <sys/resource.h>

/* X/Motif includes (from Ken Chiu)  */

#include </usr/X11R6/include/Xm/Xm.h>
#include </usr/X11R6/include/Xm/DialogS.h>
#include </usr/X11R6/include/Xm/DrawingA.h>
#include </usr/X11R6/include/Xm/Form.h>
#include </usr/X11R6/include/Xm/FileSB.h>
#include </usr/X11R6/include/Xm/Frame.h>
#include </usr/X11R6/include/Xm/Label.h>
#include </usr/X11R6/include/Xm/LabelG.h>
#include </usr/X11R6/include/Xm/MessageB.h>
#include </usr/X11R6/include/Xm/PanedW.h>
#include </usr/X11R6/include/Xm/PushB.h>
#include </usr/X11R6/include/Xm/PushBG.h>
#include </usr/X11R6/include/Xm/RowColumn.h>
#include </usr/X11R6/include/Xm/SeparatoG.h>
#include </usr/X11R6/include/Xm/ScrolledW.h>
#include </usr/X11R6/include/Xm/SelectioB.h>
/*
#include </usr/LessTif/Motif2.0/include/Xm/DialogS.h>
#include </usr/LessTif/Motif2.0/include/Xm/DrawingA.h>
#include </usr/LessTif/Motif2.0/include/Xm/Form.h>
#include </usr/LessTif/Motif2.0/include/Xm/FileSB.h>
#include </usr/LessTif/Motif2.0/include/Xm/Frame.h>
#include </usr/LessTif/Motif2.0/include/Xm/Label.h>
#include </usr/LessTif/Motif2.0/include/Xm/LabelG.h>
#include </usr/LessTif/Motif2.0/include/Xm/MessageB.h>
#include </usr/LessTif/Motif2.0/include/Xm/PanedW.h>
#include </usr/LessTif/Motif2.0/include/Xm/PushB.h>
#include </usr/LessTif/Motif2.0/include/Xm/PushBG.h>
#include </usr/LessTif/Motif2.0/include/Xm/RowColumn.h>
#include </usr/LessTif/Motif2.0/include/Xm/SeparatoG.h>
#include </usr/LessTif/Motif2.0/include/Xm/ScrolledW.h>
#include </usr/LessTif/Motif2.0/include/Xm/SelectioB.h>
#include </usr/LessTif/Motif2.0/include/Xm/Xm.h>
*/

#include <X11/cursorfont.h>

/* local includes */
#include <hbmat.h>
#include <hbdraw.h>

#endif 
