#include <hb.h>
/* prompts */
#define FILE_PROMPT "Enter File Name"
#define PRINT_PROMPT "Enter Print Command"
/* button titles */
#define FILE_BUTTON "Print To File"
#define COMMAND_BUTTON "Print Using Command"

/* globals for the print dialog are in globals.c */
extern Widget print_dialog;
/* extern int print_creator; */
/* end of globals/extern */

/* based on the modify_btn program in Vol 6. of the O'Reilly X Series.
 * Started on 1/13/95 by Tom Loos
 */
void PrintDialog(Widget parent, int zc)
{
    XmString t = XmStringCreateSimple(FILE_PROMPT);
    XmString button1 = XmStringCreateSimple(FILE_BUTTON);
    XmString button2 = XmStringCreateSimple(COMMAND_BUTTON);
    int user_data;
    Arg args[3];
    Widget toggle_box;

    
    /* handle old dialog */
    /*if (print_dialog != NULL)
     *	XtDestroyWidget(print_dialog);
     */

    /* user_data is of the form: xxxxy, where xxxx = zoom count and 
     *  setting of print button 
     */
    user_data = zc * 2;
    
    if (RCAS[zc].mapped == 0)
    {
	fprintf(stderr,"Window unmapped -- print failed\n");
	return;
    }
    
        
    /* create print dialog */
    XtSetArg(args[0], XmNselectionLabelString, t);
    XtSetArg(args[1], XmNautoUnmanage, False);
    XtSetArg(args[2], XmNuserData, user_data);

    print_dialog = XmCreatePromptDialog(parent, 
					"Print Matrix",
					args, 3);


    /* free the prompt string */
    XmStringFree(t);
    
    /* When user types name/command, do some work */
    XtAddCallback(print_dialog, XmNokCallback, print_callback, parent);
    
    /* If user calls cancel, destroy the dialog */
    XtAddCallback(print_dialog, XmNcancelCallback, print_cancel_callback, 
		  parent);

    
    /* No help */
    XtUnmanageChild(XmSelectionBoxGetChild(print_dialog,
					   XmDIALOG_HELP_BUTTON));
    /* or application */
    XtUnmanageChild(XmSelectionBoxGetChild(print_dialog,
					   XmDIALOG_APPLY_BUTTON));
    
    /* create radio buttons */
    toggle_box = XmVaCreateSimpleRadioBox( 
	print_dialog, /* parent */
	"print_box", user_data, /* init val */
	print_toggle_callback,
	XmVaRADIOBUTTON, button1, 0, NULL, NULL,
	XmVaRADIOBUTTON, button2, 0, NULL, NULL,
	NULL);
    XtManageChild(toggle_box);
    
    /* and show the dialog */
    XtManageChild(print_dialog);
    XtPopup(XtParent(print_dialog), XtGrabNone);
}

void print_cancel_callback(Widget widget, XtPointer client_data, XtPointer call_data)
{
    /* set creator to null */
    /* print_creator = -1; */
    /* and kill the dialog */
    XtDestroyWidget((Widget) widget);
}

void print_callback(Widget widget, XtPointer client_data, XtPointer call_data)
{
    int n;
    char *text;
    int zc;
    int rc;
    int screen_cols, screen_rows;
    int pipefd;
    FILE *outfp;

    /* get user data */
    XtVaGetValues(widget, XmNuserData, &n, NULL);
    /* decode user data */
    /* zc is stored in all bits but LSB */
    zc = n >> 1;
    /* n (setting of flag) is LSB */
    n = n % 2;

    /* get text of selection box */
    XmStringGetLtoR(((XmSelectionBoxCallbackStruct *) call_data)->value,
		    XmSTRING_DEFAULT_CHARSET, 
		    &text);

    /* malloc out storage for the output array */
    
    if (RCAS[zc].UsedColPtns <= 1)
	screen_cols = RCAS[zc].comp->NumCols;
    else
	screen_cols = RCAS[zc].comp->NumCols + RCAS[zc].UsedColPtns - 1;

    if (RCAS[zc].UsedRowPtns <= 1)
	screen_rows = RCAS[zc].comp->NumRows;
    else
	screen_rows = RCAS[zc].comp->NumRows + RCAS[zc].UsedRowPtns - 1;
    RCAS[zc].screen_array = (int *) malloc( screen_cols * screen_rows * 3 *
					    sizeof(int));

    /* if we got the memory */
    if (RCAS[zc].screen_array != (int *) NULL)
    {
	/* call RCAS to scale the array for us */
	RCAS[zc].action = RCASscalearray;
	ReadCompressAndScaleMat( &RCAS[zc]);
	if (n == 0) /* if printing to file */
	{
	    outfp = fopen(text,"w");
	}
	else /* printing using command */
	{
	    outfp = popen(text,"w");
	    /* call popen */
	    /* free text */
	    XtFree(text);
	    text = (char *) NULL;
	}
	/* write the array */
	if (outfp == (FILE *) NULL)
	{
	    fprintf(stderr,"Unable to print due to bad file/command name\n");
	}
	else
	{
	    PrintArray(outfp, RCAS[zc].screen_array, screen_cols, screen_rows,
		       text,zc);
	}
	/* free up the temp storage */
	free(RCAS[zc].screen_array);
	/* just in case */
	if (n == 0)
	    fclose(outfp);
	else
	    pclose(outfp);
		   
    }
    else
    {
	fprintf(stderr,"Couldn't allocate enough memory for printing\n");
    }

    /* and destroy dialog */
    XtDestroyWidget(widget);
}
void print_toggle_callback(Widget widget, XtPointer client_data, XtPointer call_data)
{
    int n, user_data;
    XmToggleButtonCallbackStruct *cbs;
    XmString str;
    Widget true_parent;

    /* get the real parent of this widget (the grandparent) */
    true_parent = XtParent(XtParent(widget));

    /* get old user data value */
    XtVaGetValues(true_parent, XmNuserData, &user_data, NULL);

    /* get input data */
    n = (int) client_data;
    cbs = (XmToggleButtonCallbackStruct *) call_data;

    /* rebuild user data */
    user_data = (user_data >> 1) * 2 + (n % 2);
    /*
     * n == 0 ==> print to file 
     * n == 1 ==> print using command 
     */
    
    /* this callback will be called for both true and false button events.
     *  Ignore false events.
     */
    if (cbs->set == False)
	return;

    /* set the prompt string according to button setting */
    if (n == 0)
	str = XmStringCreateSimple(FILE_PROMPT);
    else
    	str = XmStringCreateSimple(PRINT_PROMPT);

    /* now actually set the prompt */

    XtVaSetValues(true_parent,
		  XmNselectionLabelString, str,
		  XmNuserData, user_data, /* reset user data to reflect value */
		  NULL);
    XmStringFree(str);
}


