#include <hb.h>

/* local stuff */
#define POINTS_PER_INCH 72
#define PAPER_HEIGHT 11.0 /* in inches */
#define PAPER_WIDTH 8.5 /* in inches */

/* justification for WritePostScriptString */
#define L_JUSTIFY 0
#define C_JUSTIFY 1
#define R_JUSTIFY 2

/* main image size */
#define IMAGE_HEIGHT_IN_INCHES 5.0
/*#define TOP_TEXT_HEIGHT_IN_INCHES 1.0 */
/* #define BOT_TEXT_HEIGHT_IN_INCHES 1.0 */
#define TOP_TEXT_HEIGHT_IN_INCHES 0.6
#define BOT_TEXT_HEIGHT_IN_INCHES 0.0 

#define IMAGE_WIDTH_IN_INCHES 5.0
#define LEFT_TEXT_WIDTH_IN_INCHES 0.50
#define RIGHT_TEXT_WIDTH_IN_INCHES 1.0

#define DESIRED_HEIGHT_IN_INCHES (IMAGE_HEIGHT_IN_INCHES+TOP_TEXT_HEIGHT_IN_INCHES+BOT_TEXT_HEIGHT_IN_INCHES)
#define DESIRED_WIDTH_IN_INCHES  (IMAGE_WIDTH_IN_INCHES+LEFT_TEXT_WIDTH_IN_INCHES+RIGHT_TEXT_WIDTH_IN_INCHES)

/* font size in points */
#define EMILY_FONTSIZE 12
/* how much space do we need for each text box (in points)? */
/* top box */
#define MIN_TOP_WIDTH 40 
#define MIN_TOP_HEIGHT 72
#define TOP_COL_HEIGHT 20
#define TOP_INDICES_HEIGHT 8
#define TOP_HASH_HEIGHT 4
#define HASH_LINE_WIDTH 1
#define HASH_MARK_SIZE 8

/* right box */
#define MIN_RIGHT_WIDTH 36 
#define MIN_RIGHT_HEIGHT 40
#define RIGHT_HASH_WIDTH 4
#define RIGHT_COLS_WIDTH 10

/* left box */
#define MIN_LEFT_WIDTH 54
#define LEFT_TOTAL_WIDTH 50
#define MIN_LEFT_HEIGHT 267
#define LEFT_MAX_HEIGHT 255
#define LEFT_COLORBAR_HEIGHT 14
#define LEFT_COLORBAR_STARTWIDTH 20
#define LEFT_COLORBAR_WIDTH 30
#define LEFT_COLORBAR_SIZE USER_COLORS
#define LEFT_MIN_HEIGHT 0

/* bot box */
#define MIN_BOT_HEIGHT 72
#define MIN_BOT_WIDTH 288 /* 4 " */
#define BOT_LINE1_HEIGHT 48
#define BOT_LINE2_HEIGHT 36
#define BOT_LINE3_HEIGHT 18
#define BOT_LINE4_HEIGHT 6

void WritePostScriptHeader(FILE *outfp, char *name, 
		      int bb_left, int bb_bot, int bb_right, int bb_top)
{
    char local_name[1000];
    if (outfp == (FILE *) NULL)
    {
	return;
    }
    else
	strcpy(local_name,name);

    /* write header (based on xv output) */
    fprintf(outfp,"%%!PS-Adobe-2.0 EPSF-2.0\n");
    fprintf(outfp,"%%%%Title: %s\n",local_name);
    fprintf(outfp,"%%%%Creator: emily\n");
    fprintf(outfp,"%%%%BoundingBox: %d %d %d %d\n",bb_left,bb_bot,bb_right,
	    bb_top);
    fprintf(outfp,"%%%%Pages: 1\n");
    fprintf(outfp,"%%%%DocumentFonts: \n");
    fprintf(outfp,"%%%%EndComments \n");
    fprintf(outfp,"%%%%EndProlog \n");
    fprintf(outfp,"\n");
    fprintf(outfp,"%%%%Page: 1 1 \n");
    fprintf(outfp,"\n");
}
void WritePostScriptCode(FILE *outfp, int bb_left, int bb_bot, int desired_h,
		    int desired_w, int matrix_size)
{
    if (outfp == (FILE *) NULL)
	return;

    /* PostScript functions below:
     *  some basic setup code.
     *  colorimage
     *  startx starty hash_mark_height line_width length drawhoriz
     *  startx starty hash_mark_width line_width height drawvert
     *  string field_width centerstring
     *  string field_width rightstring
     */

    fprintf(outfp,"%% remember original state\n");
    fprintf(outfp,"/origstate save def\n\n");
    fprintf(outfp,"%% build temp dictionary\n");
    fprintf(outfp,"20 dict begin\n\n");
    fprintf(outfp,"%% define string to hold a scanline's worth of data\n");
    fprintf(outfp,"/pix %d string def\n\n",matrix_size*3);

    /* code for colorimage */
    fprintf(outfp,"%% define 'colorimage' if it isn't defined\n");
    fprintf(outfp,"%%   ('colortogray' and 'mergeprocs' come from xwd2ps\n");
    fprintf(outfp,"%%     via xgrab)\n");
    fprintf(outfp,"/colorimage where    %% do we know about 'colorimage'?\n");
    fprintf(outfp,"  { pop }            %% yes: pop off the 'dict' returned\n");
    fprintf(outfp,"  {                  %% no:  define one\n");
    fprintf(outfp,"     /colortogray {  %% define an RGB->I function\n");
    fprintf(outfp,"       /rgbdata exch store    %% call input 'rgbdata'\n");
    fprintf(outfp,"       rgbdata length 3 idiv\n");
    fprintf(outfp,"       /npixls exch store\n");
    fprintf(outfp,"       /rgbindx 0 store\n");
    fprintf(outfp,"       /grays npixls string store  %% str to hold the result\n");
    fprintf(outfp,"       0 1 npixls 1 sub { \n");
    fprintf(outfp,"         grays exch\n");
    fprintf(outfp,"         rgbdata rgbindx       get 20 mul    %% Red\n");
    fprintf(outfp,"         rgbdata rgbindx 1 add get 32 mul    %% Green\n");
    fprintf(outfp,"         rgbdata rgbindx 2 add get 12 mul    %% Blue\n");
    fprintf(outfp,"         add add 64 idiv      %% I = .5G + .31R + .18B\n");
    fprintf(outfp,"         put\n");
    fprintf(outfp,"         /rgbindx rgbindx 3 add store\n");
    fprintf(outfp,"     } for\n");
    fprintf(outfp,"     grays\n");
    fprintf(outfp,"   } bind def\n");
    fprintf(outfp,"\n");
    fprintf(outfp,"   %% Utility procedure for colorimage operator.\n");
    fprintf(outfp,"   %% This procedure takes two procedures off the\n");
    fprintf(outfp,"   %% stack and merges them into a single procedure.\n");
    fprintf(outfp,"\n");
    fprintf(outfp,"   /mergeprocs { %% def\n");
    fprintf(outfp,"     dup length\n");
    fprintf(outfp,"     3 -1 roll\n");
    fprintf(outfp,"     dup\n");
    fprintf(outfp,"     length\n");
    fprintf(outfp,"     dup\n");
    fprintf(outfp,"     5 1 roll\n");
    fprintf(outfp,"     3 -1 roll\n");
    fprintf(outfp,"     add\n");
    fprintf(outfp,"     array cvx\n");
    fprintf(outfp,"     dup\n");
    fprintf(outfp,"     3 -1 roll\n");
    fprintf(outfp,"     0 exch\n");
    fprintf(outfp,"     putinterval\n");
    fprintf(outfp,"     dup\n");
    fprintf(outfp,"     4 2 roll\n");
    fprintf(outfp,"     putinterval\n");
    fprintf(outfp,"   } bind def\n");
    fprintf(outfp,"\n");
    fprintf(outfp,"   /colorimage { %% def\n");
    fprintf(outfp,"     pop pop     %% remove 'false 3' operands\n");
    fprintf(outfp,"     {colortogray} mergeprocs\n");
    fprintf(outfp,"     image\n");
    fprintf(outfp,"   } bind def \n");
    fprintf(outfp," } ifelse          %% end of 'false' case\n\n");

    /* code for drawhoriz */

    fprintf(outfp,"/drawhoriz %% stack:width line_width max_h starty startx\n");
    fprintf(outfp,"{\n");
    fprintf(outfp,"   %% pop args from stack \n");
    fprintf(outfp,"   /w1 exch def %% get width from stack\n");
    fprintf(outfp,"   /line_width exch def %% and the line width\n");
    fprintf(outfp,"   /max_h exch def %% max line height\n");
    fprintf(outfp,"   /starty exch def %% starty	\n");
    fprintf(outfp,"   /startx exch def %% starty	\n\n");
    fprintf(outfp,"   %% local vars \n");
    fprintf(outfp,"   /half_h max_h 2 div def %% half_h = 1/2 max height\n");
    fprintf(outfp,"   /quarter_h half_h 2 div def %% quarter_h = 1/2 half height\n");
    fprintf(outfp,"   /half_width w1 2 div def %% 1/2 the width\n");
    fprintf(outfp,"   /quarter_width w1 4 div def %% 1/4 the width\n");
    fprintf(outfp,"   /quarter_way quarter_width startx add def %% 1/4 across\n");
    fprintf(outfp,"   /half_way half_width startx add def %% 1/2 across \n");
    fprintf(outfp,"   /three_fourths_way quarter_width half_width add startx add def %% 3/4 across\n");

    fprintf(outfp,"   newpath\n");
    fprintf(outfp,"   startx starty moveto \n");
    fprintf(outfp,"   line_width setlinewidth %% set the line width\n");
    fprintf(outfp,"   w1 0 rlineto %% draw a long line\n");
    fprintf(outfp,"   startx half_h neg starty add moveto   %% move to beginning and down 1/2\n");
    fprintf(outfp,"   0 max_h rlineto %% draw left line\n");
    fprintf(outfp,"\n");
    fprintf(outfp,"   quarter_way quarter_h neg starty add moveto %% move to  1/4 and down 1/8\n");
    fprintf(outfp,"   0 half_h rlineto %% draw 1/4 line 1/4 long\n");
    fprintf(outfp,"\n");
    fprintf(outfp,"   half_way quarter_h neg starty add moveto   %% move to middle and down 1/4\n");
    fprintf(outfp,"   0 half_h rlineto %% draw middle line 1/2 long\n");
    fprintf(outfp,"\n");
    fprintf(outfp,"   three_fourths_way quarter_h neg starty add moveto %% move to  3/4 and down 1/8\n");
    fprintf(outfp,"   0 half_h rlineto %% draw 3/4 line 1/4 long\n");

    fprintf(outfp,"   startx w1 add half_h neg starty add moveto %% go to RHS and down 1/2\n");
    fprintf(outfp,"   0 max_h rlineto %% draw right line\n");
    fprintf(outfp,"   stroke\n");
    fprintf(outfp,"} def\n\n");

    /* code for drawvert */
    fprintf(outfp,"/drawvert %%stack: height line_width max_w starty startx\n");
    fprintf(outfp,"{\n");
    fprintf(outfp,"   %% pop args from stack \n");
    fprintf(outfp,"   /h1 exch def %% get width from stack\n");
    fprintf(outfp,"   /line_width exch def %% and the line width\n");
    fprintf(outfp,"   /max_w exch def %% max line height\n");
    fprintf(outfp,"   /starty exch def %% starty	\n");
    fprintf(outfp,"   /startx exch def %% starty	\n");
    
    fprintf(outfp,"   %% local vars \n");
    fprintf(outfp,"   /half_w max_w 2 div def %% half_w = 1/2 max width\n");
    fprintf(outfp,"   /quarter_w half_w 2 div def %% quarter_w = 1/2 half width\n");
    
    fprintf(outfp,"   /half_height h1 2 div def %% 1/2 the height\n");
    fprintf(outfp,"   /quarter_height h1 4 div def %% 1/4 the hefith\n");
    fprintf(outfp,"   /quarter_way quarter_height starty add def %% 1/4 up\n");
    fprintf(outfp,"   /half_way half_height starty add def %% half way up the line\n");
    fprintf(outfp,"   /three_fourths_way quarter_height half_height add starty add def %% 3/4 up\n");

    fprintf(outfp,"   newpath\n");
    fprintf(outfp,"   startx starty moveto \n");
    fprintf(outfp,"   line_width setlinewidth %% set the line width\n");
    fprintf(outfp,"   0 h1 rlineto %% draw a long line\n");
    fprintf(outfp,"   half_w neg startx add starty moveto   %% move to beginning and left 1/2\n");
    fprintf(outfp,"   max_w 0 rlineto %% draw bottom line\n");
    fprintf(outfp,"\n");
    fprintf(outfp,"   startx quarter_w sub quarter_way moveto%% move to  1/4 and left 1/4\n");
    fprintf(outfp,"   half_w 0 rlineto %% draw 1/4 line 1/2 long\n");
    
    fprintf(outfp,"   quarter_w neg startx add half_way moveto   %% move to middle and down 1/4\n");
    fprintf(outfp,"   half_w 0 rlineto %% draw middle line 1/2 long\n");
    fprintf(outfp,"\n");
    fprintf(outfp,"   quarter_w neg startx add three_fourths_way moveto %% move to 3/4 and left 1/8\n");
    fprintf(outfp,"   half_w 0 rlineto %% draw 3/4 line 1/2 long\n");
    fprintf(outfp,"\n");
    fprintf(outfp,"   half_h neg startx add h1 starty add moveto %% go to RHS and down 1/2\n");
    fprintf(outfp,"   max_w 0 rlineto %% draw top line\n");
    fprintf(outfp,"   stroke\n");
    fprintf(outfp,"} def\n\n");

/* centerstring code */
    fprintf(outfp,"/centerstring %% stack: width string\n");
    fprintf(outfp,"{\n");
    fprintf(outfp,"   /w1 exch def  %% get width from stack\n");
    fprintf(outfp,"   /s1 exch def  %% and the string\n");
    fprintf(outfp,"   s1 stringwidth pop %% get string width\n");
    fprintf(outfp,"   w1 exch sub 2 div %% calculate center value\n");
    fprintf(outfp,"   0 rmoveto %% move there\n");
    fprintf(outfp,"   s1 show %% and draw that sucker.\n");
    fprintf(outfp,"} def\n\n");

/* rightstring code (from PostScript Tutorial) */
    fprintf(outfp,"/rightstring %% stack: width string\n");
    fprintf(outfp,"{\n");
    fprintf(outfp,"   /w1 exch def  %% get width from stack\n");
    fprintf(outfp,"   /s1 exch def  %% and the string\n");
    fprintf(outfp,"   s1 stringwidth pop %% get string width\n");
    fprintf(outfp,"   w1 exch sub %% calculate right value\n");
    fprintf(outfp,"   0 rmoveto %% move there\n");
    fprintf(outfp,"   s1 show %% and draw that sucker.\n");
    fprintf(outfp,"} def\n\n");

}
void WriteColorBar(FILE *outfp, int startx, int starty, int width, 
		   int height, int *colors)
{
    int i, color_index;

    if (outfp == (FILE *) NULL)
	return;
    if ((height <= 0) || (width <= 0))
	return;
    
    /* start at top */
    color_index = 0;
    fprintf(outfp,"gsave %% save state...\n");
    fprintf(outfp,"1 setlinewidth %% set the line width\n");
    for (i = height-1; i >= 0; i--)
    {
	fprintf(outfp,"newpath\n");
	fprintf(outfp,"%d %d moveto\n",startx,i);
	fprintf(outfp,"%f %f %f setrgbcolor\t%% set color\n",
		(float) (colors[color_index]/255.0),
		(float) (colors[color_index+1]/255.0),
		(float) (colors[color_index+2]/255.0));
	color_index += 3;
	fprintf(outfp,"%d 0 rlineto\n",width);
	fprintf(outfp,"stroke\t%% and draw a line of the colorbar...\n");
    }
    fprintf(outfp,"grestore %% done with colorbar\n");
}
void WritePostScriptBlock(FILE *outfp,int xdim,int ydim,int *array, int dup)
{
    int i,j, idx;
    int k;

    if (outfp == (FILE *) NULL)
	return;
    /* fprintf(outfp,"gsave\t\t%% save graphics state\n\n"); */
    fprintf(outfp,"%d %d 8              %% dimensions of data\n",xdim*dup,ydim);
/*    fprintf(outfp,"[%d 0 0 %d 0 %d ]    %% mapping matrix\n",xdim,-ydim,ydim); */
    fprintf(outfp,"[%d 0 0 %d 0 %d ]    %% mapping matrix\n",xdim*dup,-ydim,ydim); 
    
    fprintf(outfp,"{currentfile pix readhexstring pop}\n");
    fprintf(outfp,"false 3 colorimage\n");

    idx = 0;
    for (i = 0; i < ydim; i++)
    {
	for (j = 0; j < xdim; j++)
	{
	    for (k = 0; k < dup; k++)
	    {
		fprintf(outfp,"%02x%02x%02x",array[idx],array[idx+1],array[idx+2]);
	    }
	    idx += 3;
	    if (j % 10 == 9)
		fprintf(outfp,"\n");
	}
	fprintf(outfp,"\n\n\n");
    }
    /* fprintf(outfp,"grestore\t\t%% restore to initial state\n\n"); */
}
void WritePostScriptFooter(FILE *outfp)
{
    if (outfp == (FILE *) NULL)
    {
	return;
    }
    fprintf(outfp,"showpage\n\n");
    fprintf(outfp,"%% stop using temporary dictionary\n");
    fprintf(outfp,"end\n\n");
    fprintf(outfp,"%% restore original state\n");
    fprintf(outfp,"origstate restore\n\n");
    fprintf(outfp,"%%%%Trailer\n");
}
void SetupPostScriptFont(FILE *outfp, int fontsize)
{
    if (outfp == (FILE *) NULL)
	outfp = stdout;
    fprintf(outfp,"/Times-Roman findfont %% get the basic T.R. font\n");
    fprintf(outfp,"%d scalefont          %% scale the font to %d points\n",
	    fontsize,fontsize);
    fprintf(outfp,"setfont               %% make it the current font\n");
}

/*
 * centering controlled by center_width parm -- if center_width > 0,
 * string will be centered.
 * 
 */
void WritePostScriptString(FILE *outfp, int text_left, int text_bot, 
			   char *s, int justify, int width)
{
    if (outfp == (FILE *) NULL)
	outfp = stdout;
    /* fprintf(outfp,"newpath          %% start a new path\n"); */
    fprintf(outfp,"%d %d moveto    %% set lower left corner of text\n",
	    text_left, text_bot);
    if (justify == L_JUSTIFY) /* just draw it */
	fprintf(outfp,"(%s) show       %% typeset string\n",s);
    else
    {
	if (width <= 0)
	{
	    fprintf(stderr,"BUG: try to write string of %d width\n",width);
	    return;
	}
	if (justify == C_JUSTIFY) /* center it */
	    fprintf(outfp,"(%s) %d centerstring       %% typeset string\n",
		    s,width);
	else if (justify == R_JUSTIFY) /* right justify it */
	    fprintf(outfp,"(%s) %d rightstring       %% typeset string\n",
		    s,width);
	else
	    fprintf(stderr,"BUG: invalid justification %d\n",justify);
    }
}

void WriteTopTextBox( FILE *outfp, int bb_left, int bb_bot, int width, 
		      int height, char *key, int mincol, int maxcol, 
		      int minrow, int maxrow)
{
    /* 
     * Layout of text box:
     *
     *  Rows 60-71: title
     *  Rows 40-59: blank space
     *  Rows 28-39: "Col" left justified
     *  Rows 16-27: mincol, maxcol values 
     *  Rows 12-15: blank
     *  Rows 4-11:  horiz hash marks
     *  Rows 0-3: blank
     */

    char str[1000];

    /* make sure we have an output file */
    if (outfp == (FILE *) NULL)
	return;
    /* and enough width and height */
    if ((width < MIN_TOP_WIDTH) || (height < MIN_TOP_HEIGHT))
	return;

    /* setup fonts */
    SetupPostScriptFont( outfp, EMILY_FONTSIZE);
    /* ok, now move to top and write centered title string */
/*
 *   sprintf(str,"%s Matrix Printout from emily",key);
 *   WritePostScriptString( outfp, bb_left, 
 *			   bb_bot+MIN_TOP_HEIGHT-EMILY_FONTSIZE,
 *			   str,  C_JUSTIFY, width);
 */
    /* now do the column thing */
    WritePostScriptString( outfp, bb_left, bb_bot+TOP_COL_HEIGHT,
			   "Columns", L_JUSTIFY, width);

    /* now the column indices */
    sprintf(str, "%d",mincol);
    WritePostScriptString( outfp, bb_left, bb_bot+TOP_INDICES_HEIGHT,
			   str, L_JUSTIFY, width);

    sprintf(str, "%d",maxcol);
    WritePostScriptString( outfp, bb_left, bb_bot+TOP_INDICES_HEIGHT,
			   str, R_JUSTIFY, width);

    /* finally, the hash marks */
    fprintf( outfp, "%d %d %d %d %d drawhoriz\n",
	     bb_left, bb_bot+TOP_HASH_HEIGHT, HASH_MARK_SIZE, 
	     HASH_LINE_WIDTH, width);
    
}

void WriteRightTextBox( FILE *outfp, int bb_left, int bb_bot, 
			int width, int height,
		        int minrow, int maxrow)
{
    char str[1000];

    /* check input */
    if (outfp == (FILE *) NULL)
	return;
    if ((width < MIN_RIGHT_WIDTH) || (height < MIN_RIGHT_HEIGHT))
	return;

    /* 
     * Right box layout
     * Cols 1-4: blank
     * Cols 5-12: right hash marks
     * Cols 13-36: Row numbers
     */

    /* first the hash marks */
    fprintf( outfp, "%d %d %d %d %d drawvert\n",
	     bb_left+4, bb_bot, HASH_MARK_SIZE, 
	     HASH_LINE_WIDTH, height);
    /* then the row lines */

    /* min row goes at top... */
    WritePostScriptString( outfp, bb_left+RIGHT_COLS_WIDTH, 
			   bb_bot+height,
			   "Rows", L_JUSTIFY, width);

    sprintf(str, "%d",minrow);
    WritePostScriptString( outfp, bb_left+RIGHT_COLS_WIDTH, 
			   bb_bot+height-EMILY_FONTSIZE,
			   str, L_JUSTIFY, width);

    /* max row goes at bottom */
    sprintf(str, "%d",maxrow);
    WritePostScriptString( outfp, bb_left+RIGHT_COLS_WIDTH, bb_bot,
			   str, L_JUSTIFY, width);

    
}

void WriteLeftTextBox( FILE *outfp, int bb_left, int bb_bot,
		   int width, int height, double min, double max)
		
{
    /* cols 1-50: colorbar 
     *       51-54: blank
     */
    char str[1000];
    /* check input */
    if (outfp == (FILE *) NULL)
	return;
    if ((width < MIN_LEFT_WIDTH) || (height < MIN_LEFT_HEIGHT))
	return;
    
    /* draw the text */
    sprintf(str,"%7.1e",min);
    WritePostScriptString( outfp, bb_left, bb_bot,
			   str, R_JUSTIFY, LEFT_TOTAL_WIDTH);

    sprintf(str,"%7.1e",max);
    WritePostScriptString( outfp, bb_left, bb_bot+LEFT_MAX_HEIGHT,
			   str, R_JUSTIFY, LEFT_TOTAL_WIDTH);

    /* colorbar moved to print array */
}
void WriteBotTextBox( FILE *outfp, int bb_left, int bb_bot,
		      int width, int height, 
		      double in_min, int minrow, int mincol,
		      double in_max, int maxrow, int maxcol)
{
    char str[1000];
    if (outfp == (FILE *) NULL)
	return;
    if ((width < MIN_BOT_WIDTH) || (height < MIN_BOT_HEIGHT))
	return;
    /* line 1: input min */
    fprintf(outfp,"%% bottom text box\n");
    sprintf(str,"Input Min: %7.1e at (%7d, %7d)",in_min, 
	    minrow+STARTING_OUT_MAT_ADDR, 
	    mincol+STARTING_OUT_MAT_ADDR);
    WritePostScriptString( outfp, bb_left, bb_bot+BOT_LINE1_HEIGHT,
			   str, L_JUSTIFY, width);

    /* line 2: input max */
    sprintf(str,"Input Max: %7.1e at (%7d, %7d)",in_max, 
	    maxrow+STARTING_OUT_MAT_ADDR, 
	    maxcol+STARTING_OUT_MAT_ADDR);
    WritePostScriptString( outfp, bb_left, bb_bot+BOT_LINE2_HEIGHT,
			   str, L_JUSTIFY, width);

    /* line 3: line 1 of blurb */
    WritePostScriptString( outfp, bb_left, bb_bot+BOT_LINE3_HEIGHT,
			   "Values on color bar and printout are scaled values",
			   L_JUSTIFY, width);

    /* line 4: line 2 of blurb */
    WritePostScriptString( outfp, bb_left, bb_bot+BOT_LINE4_HEIGHT,
			   "which are NOT the same as the input values.",
			   L_JUSTIFY, width);

}
void PrintArray( FILE *outfp, int *array, int cols, int rows, char *name,
		 int zc)
{
    int bb_left, bb_bot;
    int paper_height, paper_width;
    int image_h, image_w, bb_h, bb_w;
    int max_dim;
    double height_ratio, width_ratio;
    double min, max, in_min, in_max;
    int minrow, mincol, maxrow, maxcol;
    int submat_minrow, submat_mincol; 
    int submat_maxrow, submat_maxcol; 
    int num_rows, num_cols;
    char *key;
    RGB colors[MAXIMUM_COLORS];
    int color_ints[USER_COLORS * 3];
    int src_idx, dest_idx;

    /* fprintf(stderr,"In PrintArray: cols = %d, rows %d\n",cols,rows);  */
    /* check output file to make sure it's open */
    if (outfp == (FILE *) NULL)
    {
	fprintf(stderr,"PrintArray -- output file not opened\n");
	return;
    }

    /* scale the output to conform to input size */
    /* first, get maximum of the 2 dimensions */
    if (cols > rows)
	max_dim = cols;
    else
	max_dim = rows;

    /* set up color bar mapping */
    GetColors(RCAS[zc].colormap, colors);

    dest_idx = 0;
    /* set up all the colors but the last 1 */
    for (src_idx = MAXIMUM_COLORS-1; src_idx > RESERVED_COLORS; src_idx--)
    {
	color_ints[dest_idx] = colors[src_idx].Red;
	dest_idx++;
	color_ints[dest_idx] = colors[src_idx].Green;
	dest_idx++;
	color_ints[dest_idx] = colors[src_idx].Blue;
	dest_idx++;
    }

    /* set color 0 */
    if (RCAS[zc].force_black)
    {
	src_idx = RCAS[zc].what_is_black;
    }
    else
    {
	src_idx = RESERVED_COLORS;
    }
    color_ints[dest_idx] = colors[src_idx].Red;
    dest_idx++;
    color_ints[dest_idx] = colors[src_idx].Green;
    dest_idx++;
    color_ints[dest_idx] = colors[src_idx].Blue;
    dest_idx++;
    /* fprintf(stderr,"final dest_idx = %d\n",dest_idx); */

    /* calculate ratios of height and width wrt maximum dimension */
    height_ratio = (double) rows/ (double) max_dim;
    width_ratio = (double) cols/ (double) max_dim;

    /* center box */
    paper_height = (int) (POINTS_PER_INCH * PAPER_HEIGHT); /* ydim */
    paper_width = (int) (POINTS_PER_INCH * PAPER_WIDTH); /* xdim */

    /* desired height and width are 5''*ratios on the page */
    image_h = (int) (IMAGE_HEIGHT_IN_INCHES * POINTS_PER_INCH *
		       height_ratio);
    image_w = (int) (IMAGE_WIDTH_IN_INCHES * POINTS_PER_INCH * 
		       width_ratio);
    
    /* calculate the bounding box heights and widths */
    bb_h = (int) (DESIRED_HEIGHT_IN_INCHES * POINTS_PER_INCH *
		       height_ratio);
    bb_w = (int) (DESIRED_WIDTH_IN_INCHES * POINTS_PER_INCH * 
		       width_ratio);
    bb_left = (paper_width - bb_w) / 2;
    if (bb_left < 0)
	bb_left = 0;
    bb_bot = (paper_height - bb_h) / 2;
    if (bb_bot < 0)
	bb_bot = 0;

/*
 * fprintf(stderr,"ph = %d, des_h = %d, bb_bot = %d, pw = %d, des_w = %d, bb_left = %d\n",
 *	paper_height, bb_h, bb_bot, 
 *	paper_width, bb_w, bb_left);
 */

    if (name != (char *) NULL)
    {
	WritePostScriptHeader(outfp,name,bb_left,bb_bot,bb_left+bb_w,
			      bb_bot+bb_h);
    }
    else
    {
	WritePostScriptHeader(outfp,"stdout",bb_left,bb_bot,bb_left+bb_w,
			      bb_bot+bb_h);
    }
   /* fprintf(stderr,"after WritePostScriptHeader\n"); */
    
    /* write out setup and utility code */
    WritePostScriptCode(outfp,bb_left, bb_bot, image_h, image_w,max_dim);
    /* fprintf(stderr,"after WritePostScriptCode\n"); */

    /* get RCAS stats */
    GetMaxMinEtc( &(RCAS[zc]), &min, &max, &in_min, &in_max,
		  &minrow, &mincol, &maxrow, &maxcol,
		  &submat_minrow, &submat_mincol,&submat_maxrow,&submat_maxcol,
		  &num_rows, &num_cols, &key);

    /* set up for drawing top box */
    /* lower left corner is 0,0 */
    /* if want to move LL corner: x y translate */
    fprintf(outfp,"%% lower left corner for top box\n");
    /* move above image (and past left side) */
    fprintf(outfp,"%d %d translate\n\n",bb_left+MIN_LEFT_WIDTH,
	    bb_bot+image_h/*+MIN_BOT_HEIGHT*/);


    WriteTopTextBox(outfp, /*bb_left,  */
		    0,
		    /*(int) (IMAGE_HEIGHT_IN_INCHES*POINTS_PER_INCH),  */
		    0,
		    image_w, MIN_TOP_HEIGHT, key, 
		    submat_mincol, submat_maxcol, 
		    submat_minrow, submat_maxrow);

    /* image stuff */
    fprintf(outfp,"%% reset lower left corner for right box\n");
    /* move down to right box LL corner (down and right across image) */
    fprintf(outfp,"%d %d neg translate\n\n",image_w,image_h);
    /* draw right box */
    WriteRightTextBox(outfp, 0, 0, MIN_RIGHT_WIDTH, image_h, 
		      submat_minrow, submat_maxrow);

    fprintf(outfp,"%% reset lower left corner for left side \n");
    /* move down to image LL corner (left from the image) */
    fprintf(outfp,"%d neg %d translate\n\n",image_w+MIN_LEFT_WIDTH,0);
    WriteLeftTextBox( outfp, 0, 0, MIN_LEFT_WIDTH, MIN_LEFT_HEIGHT,
		   in_min, in_max); 
    /* move to draw colorbar */
    fprintf(outfp, "%% translate to beginning of colorbar\n");
    fprintf(outfp, "%d %d translate\n",LEFT_COLORBAR_STARTWIDTH, 
	    LEFT_COLORBAR_HEIGHT);

    /* draw colorbar */
    WriteColorBar(outfp, 0, 0, LEFT_COLORBAR_WIDTH, LEFT_COLORBAR_SIZE, 
		  color_ints);

    /* translate for bottom box */
    fprintf(outfp,"%% reset lower left corner for bottom text\n");
    /*  translate up from bottom box */
    fprintf(outfp,"%d %d neg translate\n\n",
	    MIN_LEFT_WIDTH-LEFT_COLORBAR_STARTWIDTH,
	    LEFT_COLORBAR_HEIGHT+MIN_BOT_HEIGHT);
/*    
 *   WriteBotTextBox(outfp, 0, 0, MIN_BOT_WIDTH, MIN_BOT_HEIGHT,
 *		    in_min, minrow, mincol,
 *		    in_max, maxrow, maxcol);
 */
    /* handle main image */
    fprintf(outfp,"%% finally, reset lower left corner for image\n");
    /*  translate up from bottom box */
    fprintf(outfp,"0 %d translate\n\n", MIN_BOT_HEIGHT);
    /* scale for image */
    fprintf(outfp,"%% size of image on paper in 1/72 inch coords\n");
    fprintf(outfp,"%d %d scale\n\n",image_w, image_h);

    /* draw the picture! */
    WritePostScriptBlock(outfp,cols,rows,array,1);

    /* outta here */
    WritePostScriptFooter(outfp);
}

