#ifdef HANDLE_PTNS
/* #define FILE_DUMP */

#include <hb.h>
/*
 * This handles calculation of a screen partition from an input partition
 *
 * An input partition is based on the size of the input matrix.  A screen
 * partition is based on the number of cols/rows in the screen (the 
 * screen_dimension parameter) and the window of the input we're interested
 * in.  Num_ptns is initially the number of input partitions, but will be 
 * updated to the number of actual partitions in viewing window screen
 *
 * Assumptions: THE INPUT PARTITION ARRAY IS SORTED!!!
 *
 * Return Value: the actual number of partitions visibile on the screen.
 *   new_ptn[] holds the screen indices of the partitions and num_ptns
 *   will be updated.
 *
 * Started: 9/22/94 by tjl
 *
 * Modified: 11/1/95 -- added useful num ptns
 */
int  CalcScreenPtn( int ptn[], int new_ptn[], int *num_ptns, int in_start, 
		    int in_end, int screen_dimension)
{
    int tmp, delta_input_dim, i, delta;
    double dim_count;
    int first_ptn, last_ptn, orig_num_ptns;
    int useful_num_ptns;

/*
 *    fprintf(stderr,"IN CSP: num_ptns = %d, in_start = %d, in_end = %d, screen_dim = %d\n",
 *	    *num_ptns,in_start,in_end,screen_dimension);
 */
    /* if only one different index or no partitions, return no partitions */
    if ((in_start == in_end) || (*num_ptns <= 1))
    {
	*num_ptns = 0;
	return;
    }
    
    /* assure in_start < in_end */
    if (in_start > in_end)
    {
	tmp = in_start;
	in_start = in_end;
	in_end = tmp;
    }
    delta_input_dim = in_end - in_start + 1;
    dim_count = ((double) screen_dimension) / ((double) delta_input_dim);
    orig_num_ptns = *num_ptns;
    /* assume C addressing for everything but the ptn vector */
    /* delta is the correction factor */
    delta = ptn[0];
    /* find first and last partition */
    first_ptn = -1; /* 10/5 -- assign first a bad val */
    for (i = 0; i < orig_num_ptns; i++)
    {
/*
 * fprintf(stderr,"in_start = %d, in_end = %d\n",in_start,in_end);
 * fprintf(stderr,"ptn[%d] = %d, ptn[%d] = %d\n",i,ptn[i]-delta,i+1,ptn[i+1]-delta);
 */

	/* see if start is in the index range of ptn i */
	if (((ptn[i]-delta) <= in_start) && ((ptn[i+1]-delta) > in_start))
	{
	    first_ptn = i;
	}

	/* and now see if end is in the index range of ptn i */
	if (((ptn[i]-delta) <= in_end) && ((ptn[i+1]-delta) > in_end))
	{
	    last_ptn = i;
	    break;
	}
    }
    if (i >= orig_num_ptns)
	last_ptn = orig_num_ptns;

    /* 10/5/94 assure first_ptn is assigned -- default is last_ptn */
    if (first_ptn == -1)
	first_ptn = last_ptn;

  /* fprintf(stderr,"first_ptn = %d, last_ptn = %d\n",first_ptn, last_ptn); */
    /* calculate how many ptns. we went through */

    *num_ptns = last_ptn - first_ptn + 1;
    /* 10/5/94 -- better check for 0 ptns */
    if (*num_ptns <= 0)
    {
	*num_ptns = 0;
	return(*num_ptns);
    }

    /* have lotsa ptns (at least 1), so assign end points */
    new_ptn[0] = 0;
    new_ptn[*num_ptns] = screen_dimension;
    useful_num_ptns = *num_ptns;
    /* assign intermediate values */
    for (i = 1; i < *num_ptns; i++)
    { 
	new_ptn[i] = (int) (((ptn[first_ptn+i]-delta) - in_start) * dim_count);
/*	fprintf(stderr,"new_ptn[%d] = %d\n",i,new_ptn[i]); */
	/* 12/1/95 */
	if (new_ptn[i] == new_ptn[i-1])
	  useful_num_ptns--;
	
    }
/*     fprintf(stderr,"new_ptn[%d] = %d\n",i,new_ptn[i]); */
    /* outta here -- return number of partitions */
    /*
     *if (useful_num_ptns != *num_ptns)
     *{
     *   printf("num ptns = %d, # of useful ptns = %d\n",*num_ptns, useful_num_ptns);
     *}
     */

    /* return(*num_ptns);*/
    return(useful_num_ptns);
}


/*
 * File: scale.c
 *
 * References:
 *  Users' Guide for the Harwell-Boeing Sparse Matrix Collection (Release I),
 *     Duff, Grimes, and Lewis, publisher?, Oct. 1992
 *
 *  Matlab External Interface Guide, The Math Works, Natick, Mass, Aug. 1992
 * 
 *  Volume I: Xlib Programming Manual, Adrian Nye, O'Reilly and Associates,
 *    7th printing, 12/91
 *
 * Includes: ScaleMat
 *
 * Started: 9/22/94 -- from original scale.c to handle ptn lines
 *
 * Modified: 
 *
 */
int ScaleMat( DenseMatrix *comp, Pixmap p, ScaleRequest *s)
{
    int scanline, col, idx, curcolor;
    int color, ncols, nrows, numcolors, startcolor, black, startx;
    int overwrite_row, overwrite_col;
    int num_row_ptns; /* number of row partitions */
    int num_col_ptns; /* number of column partitions */
    int *NewRowPtn; /* the row partition vector */
    int *NewColPtn; /* the col partition vector */
    int start_row; /* starting row of input */
    int end_row; /* ending row of input */
    int start_col; /* starting col of input */
    int end_col; /* ending col of input */
    int act_col_ptns, act_row_ptns;
    int i;
    int j; /* debugging only */
    XColor debug_color;

    char debugstr[100];
    ScaleStrategyParameters parms;
    int CurColPtn, CurRowPtn;
    double FudgeFactor = 0.1;
#ifdef FILE_DUMP    
    FILE *debugfp;

    if ((debugfp = fopen("scale_dump","w")) != NULL)
	fprintf(debugfp,"Scale dump started: min, max = %f to %f: absmin, absmax = %f to %f, numcolors = %d\n",s->min, s->max,s->absmin,s->absmax,s->numcolors);
#endif

    /* pull the data from the scalereq */
    num_row_ptns = s->num_row_ptns;
    num_col_ptns = s->num_col_ptns;
    NewRowPtn = s->row_ptn;
    NewColPtn = s->col_ptn;
    start_row = s->start_row; 
    end_row = s->end_row; 
    start_col = s->start_col;
    end_col = s->end_col; 
/* 
 *  fprintf(stderr,"IN scale2.c, num_row_ptns = %d, num_col_ptns = %d\n",
 *	    num_row_ptns, num_col_ptns);
 */
    TRACE("ScaleMat: entered!\n");
    
    /* check input args -- need check for pixmap */
    if (!(comp->one_d) || !(comp->row_major))
    {
	error("ScaleMat: comp is not a 1D row major matrix -- BUG!!",ARGERR);
	return(BAD);
    }

    /* fill out parms */
    parms.min = s->min;
    parms.max = s->max;
    parms.absmin = s->absmin;
    parms.absmax = s->absmax;
    /* up the min/max values if needed for overwrite vals */
    if ((num_row_ptns > 1) || (num_col_ptns > 1))
    {
	parms.max = parms.max + FudgeFactor* (parms.max - parms.min);
	parms.absmax = parms.absmax + FudgeFactor* (parms.absmax - parms.absmin);
    }
    /* the == is right -- only if it's 0 */
    if (parms.absmin == 0.0)
	parms.lmin = -MAX_SCALING_EXPONENT*log(10); /* say it's 10**-300 */
    else
	parms.lmin = log(parms.absmin);

    if (parms.absmax == 0.0)
	parms.lmax = -MAX_SCALING_EXPONENT*log(10); /* say it's 10**-300 */
    else
	parms.lmax = log(parms.absmax);


    /* if using log10 scaling */
    if (s->log10 != 0)
    {
	/* set the parms to log10 scaling */
	parms.log10 = 1;
	/* and downsize the min/max log values */
	parms.lmin = parms.lmin * ONE_OVER_LN_10;
	parms.lmax = parms.lmax * ONE_OVER_LN_10;
    }
    else
	parms.log10 = 0;
/*
 *fprintf(stderr,"lmin = %f, lmax = %f, log10 = %d\n",parms.lmin,parms.lmax,
 *	parms.log10);
 */
    /* reset to true absmin for linear absolute scaling if zero seen in input*/
    if (s->zero_in_input)
	parms.absmin = 0.0; 
    parms.bitmap = s->bitmap;
    parms.numcolors = numcolors = s->numcolors;

    
    if (num_col_ptns > 1)
    {
	/* 12/1/95 */
	act_col_ptns = 1;
	for (i = 1; i < num_col_ptns; i++)
	{
	    if (NewColPtn[i] != NewColPtn[i-1])
	      act_col_ptns++;
	}
	/* ncols = comp->NumCols + num_col_ptns - 1 ;*/
	ncols = comp->NumCols + act_col_ptns - 1 ;
    }
    else
	ncols = comp->NumCols;
	
    if (num_row_ptns > 1)
    {
	/* 12/1/95 */
	act_row_ptns = 1;
	for (i = 1; i < num_row_ptns; i++)
	{
	    if (NewRowPtn[i] != NewRowPtn[i-1])
	      act_row_ptns++;
	}
	nrows = comp->NumRows + act_row_ptns - 1;
    }
    else
	nrows = comp->NumRows;


    /*
     * fprintf(stderr,"Scale2: compNumRows = %d, compNumCols = %d,nrows = %d, ncols = %d\n",comp->NumRows, comp->NumCols,nrows,ncols);
     */

    startcolor = s->startcolor;
    black = s->what_is_black;
    idx = 0;
    sprintf(debugstr,"ScaleMat: before main loop! start, num = %d, %d\n",startcolor,numcolors);
    DEBUG2(debugstr);
    sprintf(debugstr,"nrows = %d, ncols = %d, bitmap = %d\n",nrows,ncols,s->bitmap);
    DEBUG2(debugstr);

    /* see if we need to do any partitioning */
/*
 *fprintf(stderr,"NCP = %d, NRP = %d\n",num_row_ptns, num_col_ptns);
 *fprintf(stderr,"start row = %d, end row = %d... start col = %d, end col = %d\n",start_row, end_row,start_col,end_col);
 */


    /* 9/22 - duplicate code for speed */
    if ((num_col_ptns <= 1) && (num_row_ptns <= 1))
    {
 /* fprintf(stderr,"No Partitions Please!\n"); */
	for (scanline = 0; scanline < nrows; scanline++)
	{
#ifdef FILE_DUMP
	    fprintf(debugfp,"\nOn Scanline %d:\n%4d: ",scanline,0);
#endif
/* init current line to start at left and unknown color */
	    startx = 0; curcolor = -1;
	    for (col = 0; col < ncols; col++)
	    {
		parms.val = comp->data.d1[ idx ];
		/* update index */
		idx++;
	    
		/* handle zero --- see hb.h for definition of SCALE_EPSILON */
		if (fabs(parms.val) < SCALE_EPSILON)  
		{
		    if (s->force_black)
			color = black - startcolor;
		    else
			color = 0; /* hmm...*/
		}
		/* call strategy and clamp colors */
		else 
		{
		    color = s->strat(&parms);
		    if (color < 0)
		    {
			fprintf(stderr,"BUG: adjusting color from %d to 0\n",color);
			fprintf(stderr,"min = %e, max = %e, val = %e, numcolors = %d\n",
				parms.min, parms.max, parms.val,parms.numcolors);
			color = 0;
		    }
		    else if (color > numcolors)
		    {
			fprintf(stderr,"BUG: adjusting color from %d to %d\n",color, numcolors);
			fprintf(stderr,"min = %e, max = %e, val = %e\n",
				parms.min, parms.max, parms.val);
			color = numcolors;
		    }
		}
#ifdef FILE_DUMP
		fprintf(debugfp,"%9.1e %3d ",parms.val, color);
		if ((col % 8) == 0 && (col != 0))
		    fprintf(debugfp,"\n%4d: ",col+1);
#endif	
		/* draw it */
			/* XDrawPoint( s->display, p, s->gcs[color+startcolor], 
			 *		 col, scanline ); 
			 */
		if (curcolor == (-1))
		{
		    curcolor = color+startcolor;
		}
		else if (curcolor != color+startcolor)
		{
		    XDrawLine(s->display, p, s->gcs[curcolor], startx, scanline,
			      col-1,scanline);
		    startx = col;
		    curcolor = color+startcolor;
		}
	    } /* end of col loop */

	    /* finish off line */
	    XDrawLine(s->display, p, s->gcs[curcolor], startx, scanline,
		      col-1,scanline);

	} /* end of scanline loop */
	TRACE("ScaleMat: exit!\n");

#ifdef FILE_DUMP
	fprintf(debugfp,"\n");
	fclose(debugfp);
#endif
	return(OK);
    } /* end of if no partitions */

    /* WE HAVE PARTITIONS */
    CurRowPtn = CurColPtn = 0;
    if (num_row_ptns <= 1) /* only have col ptns, I guess */
    {
	NewRowPtn[0] = 0;
	NewRowPtn[1] = nrows;
    }
    if (num_col_ptns <= 1) /* only have row ptns, I guess */
    {
	NewColPtn[0] = 0;
	NewColPtn[1] = ncols;
    }
/*    
 *    for (i = 0; i <= num_row_ptns; i++)
 *	printf("scale2: NewRowPtn[%d] = %d\n",i,NewRowPtn[i]);
 *
 *   for (i = 0; i <= num_col_ptns; i++)
 *	printf("scale2: NewColPtn[%d] = %d\n",i,NewColPtn[i]);
 */

    /* for all rows (scanlines) on the screen */
    for (scanline = 0; scanline < nrows; scanline++)
    {
#ifdef FILE_DUMP
	fprintf(debugfp,"\nOn Scanline %d:\n%4d: ",scanline,0);
#endif
        /* init current line to start at left and unknown color */
	startx = 0; curcolor = -1;

	/* partition stuff -- current column partition is 0 */
	CurColPtn = 0;

	/* if not on a new partition line, leave row alone */
	if (scanline != NewRowPtn[CurRowPtn+1])
	{
	    overwrite_row = 0;
	}
	else /* changing row partitions */
	{
	    /* update the row ptn */
	    overwrite_row = 1;
	    while ((NewRowPtn[CurRowPtn+1] == scanline) && (CurRowPtn < num_row_ptns))
	    {
		/* printf("Moving up CCR from %d to %d with Row = %d\n",CurRowPtn, CurRowPtn+1,scanline);*/
		CurRowPtn++;
	    }
	}
	/* now go through the cols */
	for (col = 0; col < ncols; col++)
	{
	    overwrite_col = 0;
	    /* first,see if on new col ptn */
	    if (col == NewColPtn[CurColPtn+1])
	    {
		while ((col == NewColPtn[CurColPtn+1]) && (CurColPtn < num_col_ptns))
		{
		   /*  printf("moving up CCP from %d to %d with col = %d\n",CurColPtn, CurColPtn+1,col);*/
		    CurColPtn++;
		}

		if (!overwrite_row)
		{
		    overwrite_col = 1;
		}
	    }
	    /* handle overwrite of a row/col */
	    if ((overwrite_row) || (overwrite_col))
	    {
		/* parms.val = parms.absmax; */
		/* force it to be white */
		/* 7/12/95 -- change ptn lines to be black if BG is the ptn line color */
	        if (black == PTN_LINE_COLOR)
		{
		    color = BLACK_INDEX - startcolor;
		}
		else
		{
		    color = PTN_LINE_COLOR - startcolor;
		}
	    }
	    else /* do the normal scaling */
	    {
		/* get the value */
		parms.val = comp->data.d1[ idx ];
		/* update the index into the compression matrix */
		idx++;

		/* handle zero --- see hb.h for definition of SCALE_EPSILON */
		if (fabs(parms.val) < SCALE_EPSILON)  
		{
		    if (s->force_black)
			color = black - startcolor;
		    else
			color = 0; /* hmm...*/
		}
		/* call strategy and clamp colors */
		else 
		{
		    color = s->strat(&parms);
		    if (color < 0)
		    {
			fprintf(stderr,"BUG: adjusting color from %d to 0\n",color);
			fprintf(stderr,"min = %e, max = %e, val = %e, numcolors = %d, row = %d, col = %d\n",
				parms.min, parms.max, parms.val,parms.numcolors, scanline,col);
			color = 0;
		    }
		    else if (color > numcolors)
		    {
			fprintf(stderr,"BUG: adjusting color from %d to %d\n",color, numcolors);
			fprintf(stderr,"min = %e, max = %e, val = %e\n",
			    parms.min, parms.max, parms.val);
			color = numcolors;
		    }
		}
	    }
#ifdef FILE_DUMP
	    fprintf(debugfp,"%9.1e %3d ",parms.val, color);
	    if ((col % 8) == 0 && (col != 0))
		fprintf(debugfp,"\n%4d: ",col+1);
#endif	
	    /* draw it */
	    /* XDrawPoint( s->display, p, s->gcs[color+startcolor], 
	     *		 col, scanline ); 
	     */
	    if (curcolor == (-1))
	    {
		curcolor = color+startcolor;
	    }
	    else if (curcolor != color+startcolor)
	    {
		XDrawLine(s->display, p, s->gcs[curcolor], startx, scanline,
			  col-1,scanline);
		startx = col;
		curcolor = color+startcolor;
	    }
	    /* update index */
	    /* 9/23/94 -- move this to only when not nuking a row/col */
	    /* idx++; */
	} /* end of col loop */

	/* finish off line */
	XDrawLine(s->display, p, s->gcs[curcolor], startx, scanline,
		  col-1,scanline);

    } /* end of scanline loop */
    TRACE("ScaleMat: exit!\n");

#ifdef FILE_DUMP
    fprintf(debugfp,"\n");
    fclose(debugfp);
#endif
    return(OK);

} /* end of scale mat */
	    
#endif
