/*
 * File: xdraw.c
 *
 * References:
 *  Users' Guide for the Harwell-Boeing Sparse Matrix Collection (Release I),
 *     Duff, Grimes, and Lewis, publisher?, Oct. 1992
 *
 *  Matlab External Interface Guide, The Math Works, Natick, Mass, Aug. 1992
 *
 * Professional Graphics Programming in the X Window System, Johnson and
 *     Reichard, MIS Press, 1993
 *
 * Includes: ...
 *
 * Started: ...
 *
 * Orig. Programmer: Ken Chiu (chiuk@indiana.edu)
 *
 * Modified: 2/18/94 started work to convert it to hbdraw motif code
 *           since then, this code has been contorted to do everything
 *           but draw the Mona Lisa and that'll be in the next release :-)
 *
 */
/* #include "defs.h"
#define extern
#include "globals.h"
#undef extern
*/
#include <hb.h>

void InitializeModes();
int ALLOCATED; /* has the color map been allocated? */
Colormap my_cmap;
Pixel Foreground, Background;
XColor FGColor, BGColor, DebugColor;

unsigned long allocated_colors[MAXIMUM_COLORS];
XColor new_colors[MAXIMUM_COLORS];
/* void init_to_black(); */
Arg xargs[30];
Cardinal xargcount = 0; 
/* Screen*/ int canvas_screen_num;
XtAppContext AppCon;
/* void (*CanvasInputHandler)(Event); */
extern GC foregroundGC;
/* static GC backgroundGC; */			
char outbuff[300];


/* start of code */
/* 1/30/95 -- add input_mat as a parm to undo an old hack */
/* 6/18/96 -- pass in nptns and ptn array */
int DrawMat(int argc, char *argv[], int code, DaMatrix *input_mat, 
	    int nptns, int *ptn)
{
    int i,j;
    /* DaMatrix *input_mat; */
    DaMatrix im;
    int row_major;
    int one_d;
    int copy_of_argc;
    int min_dim;
    char **copy_of_argv;
    DenseMatrix compressed_mat;
    XGCValues gcv;
    Boolean mapped;
    GC GCarray[256];
    int zoomcount; /* this is now local */
    char debugstr[200];
    int test_width, test_height,h3,w3;
    int decent_cmap_size;
    int using_def_visual;
    /* INITIALIZATIONS */
    TRACE("DrawMat called\n");
    sprintf(debugstr,"argv[0] = %s\n",argv[0]);
    DEBUG2(debugstr);

    zoomcount = 0;
    /* init the WHOLE rcas array up front */
    for (i = 0; i < MAXZOOM; i++)
    {
	InitRCASParameters( &RCAS[i] );
    }


    /* 7/21/94 init the canvas input handler to the default */
    CanvasInputHandler = DefaultInputHandler;

    /* get input matrix */
    /* commented out on 1/30/95 */
/*
 *   if (code == DRAW_MAT_USE_MATRIX)
 *   {
 *	// 
 *	// THIS IS A MAJOR LEAGUE HACK -- take the input arg, convert it
 *	// to an address and cast that address to a matrix pointer.  That
 *	// allows DrawMat() to get a matrix as input using argv.
 *	//
 *	input_mat = (DaMatrix *) atoi(argv[1]);
 *   }
 *   else
 *   {
 *	input_mat = &im;
 *   }
 */

    ALLOCATED=False;
    /*
     * Initialize the X stuff.  All the code that's still in main should
     * only have to be done for the main window.
     */


/*
 *	top_level_shell_widget = XtVaAppInitialize(
 *	&AppCon,		// application context 
 *	"HBDraw",		// class name 
 * 	NULL, 0,		// command line options stuff
 *	&argc, argv,	// argc and argv for parsing cmd line opts 
 *	NULL,			// fallback resources 
 *	XmNmappedWhenManaged, False, // varargs list for shell widget 
 *	    NULL );			
 */

    /* init X toolkit the long way from Johnson and Reichard */

    /* get the application context */
    XtToolkitInitialize();
    DEBUG2("past XtToolkit Initialize\n");
    AppCon = XtCreateApplicationContext();
    DEBUG2("past XtCreateApp..Context\n");

    /* get copy of cmd line stuff */
    copy_of_argc = argc;
    DEBUG2("copy of argc made\n");
    copy_of_argv = (char **) MALLOC(argc, sizeof(char *));
    if (copy_of_argv == (char **) NULL)
    {
	fprintf(stderr,"Unable to get a copy of the argument vector\n");
	exit(BAD);
    }
    DEBUG2("malloc'd out copy of argv\n");

    /* hmm 1 */
    bcopy( argv, copy_of_argv,argc*sizeof(char *));
    sprintf(debugstr,"post bcopy: argv[0] = %s\n",copy_of_argv[0]);
    DEBUG2(debugstr);

    /* open display */
    canvas_display = XOpenDisplay(NULL); 
    DEBUG2("opened the Display\n");
    /* 8/31/94 -- see if display was opened OK */
    if (canvas_display == (Display *) NULL)
    {
	fprintf(stderr,"Couldn't open X Display: check the DISPLAY environment variable\n");
	exit(BAD);
    }

    XtDisplayInitialize( AppCon,             /* appl. context */
			 canvas_display,     /* display */
			 NULL,               /* app name */
			 NULL,               /* app class */
			 (XrmOptionDescList) NULL,  /* options */
			 (Cardinal) 0,       /* num options */
			 ARGC_PTR &argc,
			 argv);

    DEBUG2("display initialized using XtDisplayInitialize\n");

    /* DEBUGGING MATLAB -- THE ABOVE 2 X/Xt CALLS SHOULD BE EQUIV. TO:
     * canvas_display = XtOpenDisplay( AppCon, // application context 
     *  NULL, // display name, use argv 
     *  NULL, // application name, use arg[0] 
     *  "HBDraw", // class name 
     *  (XrmOptionDescList) NULL,
     *  (Cardinal) 0, // number options 
     *  ARGC_PTR &argc, argv );
     */

    /*
     * if (canvas_display == (Display *) NULL)
     * {
     *	fprintf(stderr," Couldn't open X Display\n");
     * 	exit(BAD);
     * }
     */
    RCAS[zoomcount].used = 1; /* let's alloc. RCAS[0] */
    RCAS[zoomcount].display = canvas_display;
    RCAS[zoomcount].screen_width = MAIN_CANVAS_WIDTH; 
    RCAS[zoomcount].screen_height = MAIN_CANVAS_HEIGHT; 


    /* if want to support another/other visuals, do it here */
    /* see Johnson and Reichard pp. 440-460 */
    canvas_depth = MAXIMUM_COLORS;
    /* FDV takes cmap size as input and returns depth of cmap. */
    FindDecentVisual( canvas_display, DefaultScreen(canvas_display),
		      PseudoColor, &canvas_depth, &canvas_visual);
    if ((canvas_depth < 0) || (canvas_visual == (Visual *) NULL))
    {
	fprintf(stderr,"No PseudoColor visual with 256 colors available\n");
	exit(-1);
    }
    /* set some args... */
    xargcount= 0;
    XtSetArg(xargs[xargcount], XmNmappedWhenManaged, False); xargcount++;
    XtSetArg(xargs[xargcount], XmNallowResize, True); xargcount++;

    if (canvas_visual != DefaultVisual(canvas_display, DefaultScreen(canvas_display)))
		      
    {
	/* printf("using non-default visual! depth = %d\n",canvas_depth);  */
	using_def_visual = 0;
    }
    else
    {
	using_def_visual = 1;
    }
    XtSetArg(xargs[xargcount], XmNvisual, canvas_visual); xargcount++; 
    XtSetArg(xargs[xargcount], XmNdepth, canvas_depth); xargcount++;

   /* XtSetArg(xargs[xargcount], XmNwidth, USER_COLORS+100); xargcount++; */
   /* XtSetArg(xargs[xargcount], XmNheight, USER_COLORS+125); xargcount++;*/
    /* save old colormap and create new colormap */
    canvas_screen_num = DefaultScreen(canvas_display);
/*
 *    Default_cmap = DefaultColormap(canvas_display, 
 *				   canvas_screen_num);
 */

    /* Get current colormap and allocate color space */
    if (using_def_visual)
    {
    	my_cmap = XCopyColormapAndFree(canvas_display, 
    				       DefaultColormap( canvas_display, 
    				         DefaultScreen(canvas_display))); 
    }
    else
    {
	XFreeColormap( canvas_display,
		       DefaultColormap( canvas_display, 
					DefaultScreen(canvas_display)));

	my_cmap = XCreateColormap( canvas_display, 
				  RootWindow(canvas_display,
					     DefaultScreen(canvas_display)),
				  canvas_visual,AllocNone);
	
    } 

    if (my_cmap == (Colormap) None)
    {
	fprintf(stderr, "Error creating colormap\n");
	exit(BAD);
    }
    XtSetArg(xargs[xargcount], XmNcolormap, my_cmap); xargcount++;
    Default_cmap = my_cmap;
    DEBUG2("past colormap creation\n");



/*
 *   fprintf(stderr,"argc = %d, copy_of_argc = %d, xargcount = %d\n",
 *	    argc, copy_of_argc, xargcount);
 *   for (i = 0; i < copy_of_argc; i++)
 *	fprintf(stderr,"argv[%d] = %s\n",i,copy_of_argv[i]);
 */
    /* install cmd-line parameters */
    XtSetArg(xargs[xargcount], XmNargc, copy_of_argc); xargcount++;
    XtSetArg(xargs[xargcount], XmNargv, copy_of_argv); xargcount++;

 
    /* create top level */
    sprintf(debugstr,"trying to create the shell: xargcount = %d\n",xargcount);
    DEBUG2(debugstr);


    RCAS[zoomcount].wids.toplevel = 
	XtAppCreateShell( argv[0],
			  "hbdraw",
			  applicationShellWidgetClass,
			  canvas_display,
			  xargs, xargcount);
    
    /* toplevel = top_level_shell_widget; */
    sprintf(debugstr,"toplevel = %d\n",RCAS[zoomcount].wids.toplevel);
    DEBUG2(debugstr);

    
    XtVaGetValues(RCAS[zoomcount].wids.toplevel,
		  XmNforeground, &Foreground,
		  XmNbackground, &Background,
		  XmNmappedWhenManaged, &mapped, /* TJL 3/26 */
		  NULL);
    /* get and save foreground and background colors */
    sprintf(outbuff,"FG = %d, BG = %d\n",Foreground, Background);
    DEBUG2(outbuff);

    FGColor.pixel = Foreground;
    BGColor.pixel = Background;

    XQueryColor(canvas_display,  Default_cmap,  &FGColor);
    XQueryColor(canvas_display,  Default_cmap,  &BGColor);

    sprintf(outbuff, "Foreground color: %d, %d, %d\n",
	    FGColor.red, 
	    FGColor.green,
	    FGColor.blue);
    DEBUG2(outbuff);

    sprintf(outbuff, "Background color: %d, %d, %d\n",
	    BGColor.red, 
	    BGColor.green,
	    BGColor.blue);
    DEBUG2(outbuff);


    /* Get current colormap and allocate color space */
    GetMapping(RCAS[zoomcount].colormap, my_cmap);  
    DEBUG1("past GetMapping\n");

    /* reset foreground and background pixels with new color map */

    XtVaSetValues(RCAS[zoomcount].wids.toplevel,
		  XmNbackground, allocated_colors[BG_INDEX],
		  XmNforeground, allocated_colors[FG_INDEX],
		  NULL);

    /* XSetWindowColormap( canvas_display, canvas_window, my_cmap ); */
    

    /*
     * The form widget is for specifying how children are to be placed
     * inside.
     */

    BuildChildrenWidgets(zoomcount);

    /*
     * XtVaGetValues(toplevel,
     *		  XmNmappedWhenManaged, &mapped,
     *		  NULL);
     * fprintf(stderr,"realizing the widget: toplevel = %d (mapped = %d) \n",toplevel,mapped);
     *
     */

    XtRealizeWidget(RCAS[0].wids.toplevel);
    sprintf(debugstr,"yee haw! toplevel widget realized\n");
    DEBUG1(debugstr);
    
    /* SetXGlobals doesn't do that much anymore, but ... */
    SetXGlobals(zoomcount);
    RCAS[zoomcount].window = XtWindow(RCAS[zoomcount].wids.canvas);

    if ((RCAS[zoomcount].screen_width <= 0) || 
	(RCAS[zoomcount].screen_height <= 0) || (canvas_depth <= 0))
    {
	fprintf(stderr,"bad canvas -- aborting\n");
	exit(-1);
    }
    /* gcv.foreground = Foreground;*/
    gcv.foreground = Background;
    foregroundGC = XCreateGC(canvas_display,
			     RCAS[zoomcount].window,
			     GCForeground, &gcv);


    /* get all the GCs for drawing (in GCarray) */
    for (i = 0; i < MAXIMUM_COLORS; i++)
    {
	getGC(canvas_display,RCAS[zoomcount].window, &(GCarray[i]), 
	      allocated_colors[i]);
    }
    /* now that the GC's are initialized, we can fill out the color bar */
    RCAS[zoomcount].gcs = GCarray; /* 3/27 */
    FillColorBar(RCAS[zoomcount].wids.color_bar,zoomcount);
    sprintf(debugstr, "FCB called\n");
    DEBUG2(debugstr);

    if ((argc > 1) && (code == DRAW_MAT_USE_FILENAME)) /* add code on 3/31 */
    {
	/* only kind of matrices we can read are sparse */
	RCAS[zoomcount].action = RCASread;
	RCAS[zoomcount].file = argv[1];
#ifdef HANDLE_PTNS
	if (argc > 2)
	{
	    RCAS[zoomcount].vectfile = argv[2];
	}
#endif
	input_mat->type = HB; /* for now */
	/* get an HBMatrix -- tjl 2/28/94 */
	if ((input_mat->data.hb = (HBMatrix *) MALLOC(1, sizeof(HBMatrix))) ==
	    (HBMatrix *) NULL)
	{
	    error("couldn't allocate a matrix",MEMERR);
	    /* shouldn't get here */
	    return(BAD);
	}
    }
    else /* either input is a matrix or no input */
    {
	if (code == DRAW_MAT_USE_FILENAME) /* no input, so generate test  */
					   /*   matrix */
	{
	    int testing_size;
	    testing_size = 700;

	    input_mat->type = Dense;
	    row_major = one_d = 1;
/* commented out for timings test ONLY -- 5/11/94 */
/*	    test_width = RCAS[zoomcount].screen_width;
	    test_height = RCAS[zoomcount].screen_height;
 */

	    test_width = test_height = testing_size;
	    input_mat->data.d = GetDenseMatrix(test_width,
					       test_height,
					       row_major,one_d,0.0);

	    /* internally generated test matrix */
	    h3=test_height/3;
	    w3=test_width/3;
	    
	    for (i = 0; i < h3; i++)
		for (j = 0; j < w3; j++)
		    input_mat->data.d->data.d1[(i+h3) * test_width + (j+w3)] 
			= i+j+1;
		
	  }
	RCAS[zoomcount].action = RCAScompress;
    }
    /* matrix already read in */
    RCAS[zoomcount].input = input_mat;
    RCAS[zoomcount].comp = &compressed_mat;
    RCAS[zoomcount].pixalloc = 1;
    RCAS[zoomcount].pix = XCreatePixmap(canvas_display,
			     RCAS[zoomcount].window,
			     RCAS[zoomcount].screen_width,
			     RCAS[zoomcount].screen_height,
			     canvas_depth);
    RCAS[zoomcount].numcolors = mymin(((1<<canvas_depth)-1), (USER_COLORS-1));
    RCAS[zoomcount].startcolor = RESERVED_COLORS;
    RCAS[zoomcount].force_black = 1;
    RCAS[zoomcount].what_is_black = BLACK_INDEX;
    RCAS[zoomcount].conformal = CONFORMAL_TO_MATRIX;
    RCAS[zoomcount].cstrat = CSsum;
    /* RCAS[zoomcount].sstrat = SCabslinear; */
    RCAS[zoomcount].sstrat = SClog;
    RCAS[zoomcount].log10 = 0;
    /*RCAS[zoomcount].sstrat = SClinear2;  */
    /* clear pixmap to background color  */
    /*
     * XFillRectangle(canvas_display, RCAS[zoomcount].pix, 
     *                RCAS[zoomcount].gcs[BG_INDEX],
     *                0,0,RCAS[zoomcount].screen_width,
     *                RCAS[zoomcount].screen_height);
     */
    CanvasClearWindow(zoomcount);

    /* submatrix stuff */
    RCAS[zoomcount].input_startrow = 0;
    RCAS[zoomcount].input_startcol = 0;
    if (RCAS[zoomcount].input->type == HB)
    {
	RCAS[zoomcount].input_endrow = RCAS[zoomcount].input->hb_NumRows-1;
	RCAS[zoomcount].input_endcol = RCAS[zoomcount].input->hb_NumCols-1;
	sprintf(debugstr,"DrawMat: input_endrow, col = %d,%d: hb rows,cols = %d,%d\n",
		RCAS[zoomcount].input_endrow,RCAS[zoomcount].input_endrow, 
		RCAS[zoomcount].input->hb_NumRows,RCAS[zoomcount].input->hb_NumCols); 
	DEBUG2(debugstr);
    }
    else
    {
	RCAS[zoomcount].input_endrow = RCAS[zoomcount].input->d_NumRows-1;
	RCAS[zoomcount].input_endcol = RCAS[zoomcount].input->d_NumCols-1;
    }
#ifdef HANDLE_PTNS
    /* 6/18/96 -- check to see if row/col ptns are already in */
    if ((nptns > 0) && (ptn != (int *) NULL))
    {
	RCAS[zoomcount].RowPtn = ptn;
	RCAS[zoomcount].ColPtn = ptn;
	RCAS[zoomcount].NumRowPtns = nptns;
	RCAS[zoomcount].NumColPtns = nptns;
    }
    else
    {
	RCAS[zoomcount].RowPtn = (int *) MALLOC(MAX_PTNS, sizeof(int));
	RCAS[zoomcount].ColPtn = (int *) MALLOC(MAX_PTNS, sizeof(int));
    }
    RCAS[zoomcount].NewRowPtn = (int *) MALLOC(MAX_PTNS, sizeof(int));
    RCAS[zoomcount].NewColPtn = (int *) MALLOC(MAX_PTNS, sizeof(int));

    if ((RCAS[zoomcount].RowPtn == (int *) NULL) ||
	(RCAS[zoomcount].ColPtn == (int *) NULL) ||
	(RCAS[zoomcount].NewRowPtn == (int *) NULL) ||
	(RCAS[zoomcount].NewColPtn == (int *) NULL))
    {
	fprintf(stderr,"Partition storage unavailable--partition lines skipped\n");
	RCAS[zoomcount].vectfile = (char *) NULL;
    }
#endif 
    
    if (ReadCompressAndScaleMat( &RCAS[zoomcount] ) == OK)
    {
	DEBUG2("RCAS[zoomcount] OK: copy pix\n");
	ShowRCASResults( &RCAS[zoomcount] );
    }
    /*
     * Map all the widgets.
     */

    DEBUG2("countdown to widget mapping:3, 2, 1\n");
    /* XtRealizeWidget(toplevel); */

    XtMapWidget(RCAS[zoomcount].wids.toplevel);
    RCAS[zoomcount].mapped = 1;


	/*
	 * Enter the event loop.
	 */
    DEBUG1("Roger. We have widget mapping.  Proceed to AppMainLoop\n");
    XtAppMainLoop(AppCon);
} /* END OF DrawMat() */

void
MakeCursorPos(Widget parent, int zoomcount) {
	XmString s;
	s = XmStringCreateSimple("           ");
	RCAS[zoomcount].wids.cur_pos = XtVaCreateManagedWidget(
	    "cursor_pos",
	    xmLabelGadgetClass,
	    parent,
	    XmNlabelString, s,
	    XmNrecomputeSize, False,
	    NULL);
	
	XmStringFree(s);

}


void
MakeCanvas(Widget parent, int zoomcount) 
{
    Widget frame, canvas;
    int width, height, depth;
    String translations = "<Motion>: DrawingAreaInput()\n\
	 <Btn1Down>: DrawingAreaInput()\n\
	 <Btn1Up>: DrawingAreaInput()\n\
	 <Btn2Down>: DrawingAreaInput()\n\
	 <Btn2Up>: DrawingAreaInput()\n\
	 <Btn3Down>: DrawingAreaInput()\n\
	 <Btn3Up>: DrawingAreaInput()";
    XtTranslations t;
    Pixel fg, bg;
    char debugstr[100];

    /*
     * Create the frame around the DrawingArea widget.
     */
    
    frame = XtVaCreateManagedWidget("frame",
				    xmFrameWidgetClass, parent,
				    XmNshadowType, XmSHADOW_IN,
				    XmNshadowThickness, 4,
				    NULL);

    canvas = XtVaCreateManagedWidget("canvas",
				     xmDrawingAreaWidgetClass, frame,
				     XmNwidth, RCAS[zoomcount].screen_width,
				     XmNheight, RCAS[zoomcount].screen_height,
				     XmNdepth, canvas_depth,
				     XmNvisual, canvas_visual,
				     XmNallowResize, True,
				     XtVaTypedArg, XmNforeground, 
				     XmRString, "black", strlen("black") + 1,
				     XtVaTypedArg, XmNbackground, 
				     XmRString, "white", strlen("white") + 1,
				     NULL);

    t = XtParseTranslationTable(translations);
    sprintf(debugstr,"MakeCanvas 0: zoomcount = %d,width = %d, height = %d, depth = %d\n",
	    zoomcount,
	    RCAS[zoomcount].screen_width,
	    RCAS[zoomcount].screen_height,
	    canvas_depth);
    DEBUG2(debugstr);

    XtVaSetValues(canvas,
		  XmNtranslations, t,
		  NULL);

    /*
     * Add a callback to handle input events from the canvas area.
     */
    
    XtAddCallback(canvas, XmNinputCallback, (XtCallbackProc) input,
     NULL);
    XtAddCallback(canvas, XmNexposeCallback, (XtCallbackProc) CWexpose,
     NULL);
    XtAddCallback(canvas, XmNresizeCallback, (XtCallbackProc) CWresize,
     NULL);

    /*
     *printf("pre Getvalues of canvas: width = %d, height = %d, depth = %d\n",
     *	   width,height,depth);
     */
    XtVaGetValues(canvas,
		  XmNwidth, &width,
		  XmNheight, &height,
		  XmNdepth, &depth,
		  NULL);
    /*
     * printf("values of canvas: width = %d, height = %d, depth = %d\n",
     *	   width,height,depth); 
     */
    RCAS[zoomcount].wids.canvas = canvas;
    RCAS[zoomcount].wids.frame = frame;
}

/* rewritten 4/19/94 */
void
CanvasClearWindow(int zc) 
{
    int black;
    if (RCAS[zc].force_black)
    {
	black = RCAS[zc].what_is_black;
    }
    else
    {
	black = RESERVED_COLORS;
    }
    XFillRectangle(RCAS[zc].display, RCAS[zc].pix, RCAS[zc].gcs[black],
		   0,0,RCAS[zc].screen_width,RCAS[zc].screen_height);
}

/*
 * This is the input handler for the canvas.  First we process any mouse
 * motion events.  Then we check the global function pointer CanvasInputHandler
 * to see if anybody wants these events.
 */

void
/* CWresize(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *cbs) */
 CWresize(Widget w, XtPointer client_data, XtPointer call_data) 
 {

    int zc;

    Dimension width, height;
    Widget canvas;
    Window window;
    Display *display;
    char debugstr[100];
/*
 *   XmDrawingAreaCallbackStruct *cbs; 
 *
 *   cbs = (XmDrawingAreaCallbackStruct *) call_data;
 */

    TRACE("CWresize: entered\n");
    zc = GetZoomcount( w );
    if (zc < 0)
	return;

    /* 3/31/94 */
    if (RCAS[zc].mapped == 0)
    {
	/* fprintf(stderr,"CWresize: unmapped, so leaving\n"); */
	DEBUG1("CWresize: unmapped, so leaving\n"); 
	return;
    }
    canvas = RCAS[zc].wids.canvas;
    /* hmm tjl */
    window = RCAS[zc].window = XtWindow(w);
    display = RCAS[zc].display;

    sprintf(debugstr,"CWresize: window = %d, zc = %d\n",w,zc);
    DEBUG2(debugstr);
    XtVaGetValues(canvas,
		  XmNwidth, &width,
		  XmNheight, &height,
		  XmNdepth, &canvas_depth,
		  NULL);
    if (RCAS[zc].pixalloc == 1)
    {
	DEBUG2("FREEING PIXMAP\n");
	XFreePixmap(display, RCAS[zc].pix);
    }
    else
	RCAS[zc].pixalloc = 1; /* assume next step */

    sprintf(debugstr,"CWresize: width = %d, height = %d, depth = %d\n",width,
	   height,canvas_depth);
    DEBUG2(debugstr);
 
    RCAS[zc].pix = XCreatePixmap(display,
				 window,
				 width,
				 height,
				 canvas_depth);
	
    /* clear pixmap to background color  */

   /*
    *  XFillRectangle(display, RCAS[zc].pix, RCAS[zc].gcs[BG_INDEX],
    *	   0,0,RCAS[zc].screen_width,RCAS[zc].screen_height);
    */
    CanvasClearWindow(zc);
    RCAS[zc].action = RCAScompress;
    RCAS[zc].screen_width = width;
    RCAS[zc].screen_height = height;
    
    if (ReadCompressAndScaleMat( &RCAS[zc] ) == OK)
    {
	sprintf(debugstr,"RCAS OK: copy pix\n");
	DEBUG2(debugstr);
	ShowRCASResults( &RCAS[zc] );
    }

    TRACE("CWresize: done\n");
    return;
}


void
/* CWexpose(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *cbs)  */
 CWexpose(Widget w, XtPointer client_data, XtPointer call_data) 
{

    int zc;
    /*
     * XmDrawingAreaCallbackStruct *cbs;
     *  cbs = (XmDrawingAreaCallbackStruct *) call_data;
     */

    TRACE("CWexpose called\n");
    if ((zc = GetZoomcount(w)) < 0)
	zc = 0; /* hmm */

    RCAS[zc].action = RCASexpose;
    ShowRCASResults( &RCAS[zc] );
    TRACE("CWexpose done\n");
    return;
}
void
/* input(Widget w, XtPointer client_data, XmDrawingAreaCallbackStruct *cbs)  */
input(Widget w, XtPointer client_data, XtPointer call_data)
{
    int zc, x,y;
    int matx, maty, usablex, usabley, NumRows, NumCols, color;
    XmString s;
    char str[100];
    Event ev;
    XmDrawingAreaCallbackStruct *cbs;
    cbs = (XmDrawingAreaCallbackStruct *) call_data;
    
    if ((zc = GetZoomcount(w)) < 0)
    {
	fprintf(stderr,"input: can't find window\n");
	return;
    }
    
    /*
     * Update the cursor position in the Label widget and do calculations
     * for default button press case.
     */
    /* sprintf(str, "(%d,%d)", cbs->event->xmotion.x,
       cbs->event->xmotion.y);
       */
    /* use matrix screen pos 4/13/94 */
    x = cbs->event->xmotion.x;
    y = cbs->event->xmotion.y;

    /* make a copy of the width and height that can be altered here */
    usablex = RCAS[zc].screen_width;
    usabley = RCAS[zc].screen_height;

    /* calc. number of rows and cols */
    NumCols = IDXRANGE(RCAS[zc].input_endcol,RCAS[zc].input_startcol);
    NumRows = IDXRANGE(RCAS[zc].input_endrow,RCAS[zc].input_startrow);

    /* calc screen size */
    CalculateScreenSize(NumRows, NumCols, &usabley, &usablex, 
			RCAS[zc].conformal);

    /* figure out matrix position of current point */
    matx = RCAS[zc].input_startcol + 
	(int) (((double)x/(double)usablex) *NumCols);
    maty = RCAS[zc].input_startrow + 
	(int) (((double)y/(double) usabley) * NumRows);

    /* check for out of range values */
    if (matx > RCAS[zc].input_endcol)
	matx = RCAS[zc].input_endcol;
    else if (matx < RCAS[zc].input_startcol)
	matx = RCAS[zc].input_startcol;

    if (maty > RCAS[zc].input_endrow)
	maty = RCAS[zc].input_endrow;
    else if (maty < RCAS[zc].input_startrow)
	maty = RCAS[zc].input_startrow;
    
    if (cbs->event->xany.type == MotionNotify) 
    {
	/* this is counter intuitive to me -- it's row,col == y,x */
	/* sprintf(str, "(%d,%d)", maty, matx); */

 	/* 7/27/94 add an offset for people */
	sprintf(str, "(%d,%d)", maty+STARTING_OUT_MAT_ADDR, 
		matx+STARTING_OUT_MAT_ADDR); 
	s = XmStringCreateSimple(str);
	XtVaSetValues(RCAS[zc].wids.cur_pos, XmNlabelString, s, NULL);
	XmStringFree(s);
    }
    /*
     * If someone wants to process input events (like the zoom handler) 
     * then call it.
     */
    if (CanvasInputHandler != NULL) 
    {
	ev.zoomcount = zc;
	switch (cbs->event->xany.type) 
	{
	case MotionNotify:
	    ev.type = MOUSE_MOVEMENT;
	    ev.x = cbs->event->xmotion.x;
	    ev.y = cbs->event->xmotion.y;
	    break;
	case ButtonPress:
	    ev.type = BUTTON_PRESS;
	    ev.x = cbs->event->xbutton.x;
	    ev.y = cbs->event->xbutton.y;
	    ev.button_number = cbs->event->xbutton.button - 1;
	    break;
	case ButtonRelease:
	    ev.type = BUTTON_RELEASE;
	    ev.x = cbs->event->xbutton.x;
	    ev.y = cbs->event->xbutton.y;
	    ev.button_number = cbs->event->xbutton.button - 1;
	    break;
	}
	CanvasInputHandler(ev);
    }
}

void
MakeMsgWin(Widget parent, int zoomcount) 
{
	Widget sw;
	Dimension h;
	XmString s;

	TRACE("in MakeMsgWin\n");
	sw = XtVaCreateManagedWidget("msg_scrolled_window",
				     xmScrolledWindowWidgetClass,
				     parent,
				     XmNscrollingPolicy, XmAUTOMATIC,
				     XmNvisualPolicy, XmCONSTANT, 
				     NULL);
	s = XmStringCreateSimple("Click on an icon.");
	RCAS[zoomcount].wids.msg_label = 
	    XtVaCreateManagedWidget("msg_label", xmLabelGadgetClass,
				    sw,
				    XmNlabelString, s,
				    NULL);

	XmStringFree(s);
	
	XtVaGetValues(RCAS[zoomcount].wids.msg_label, XmNheight, &h, NULL);
	XtVaSetValues(sw, XmNheight, h + 31, NULL);
	
	TRACE("out of MakeMsgWin\n");
	/* return sw; */
	RCAS[zoomcount].wids.msg_win = sw;
}

void
SetMsgWinString(char *s, int zc) {
	XmString ms;
	char debugstr[500];

	TRACE("entering SetMsgWinString");
	sprintf(debugstr,"SetMsgWinString: zc = %d\n",zc);
	DEBUG2(debugstr);
	sprintf(debugstr,"SMWS: input string = %s\n",s);
	DEBUG2(debugstr);

	ms = XmStringCreateSimple(s);
	XtVaSetValues(RCAS[zc].wids.msg_label,
	 XmNlabelString, ms,
	 NULL);
	DEBUG2("SMWS: input string drawn\n");

	XmStringFree(ms);
	TRACE("leaving SetMsgWinString");
}

/* updated 6/2 to make the matrix min/max labels too */
void
MakeColorBar(Widget parent, int zoomcount)
{
    XmString scale, s1,s2,s3,matmin, matmax;
    int depth;

    scale = XmStringCreateSimple("Scaled Vals");
    s1 = XmStringCreateSimple("Maximum    ");
    s2 = XmStringCreateSimple("Minimum    ");
    s3 = XmStringCreateSimple("(xxxxxx,xxxxxx)");
    matmin = XmStringCreateSimple("Input Min: ");
    matmax = XmStringCreateSimple("Input Max: ");
    DEBUG2("MCB: 1\n");

    XtVaCreateManagedWidget(
	"sep_0",
	xmSeparatorGadgetClass,
	parent,
	XmNorientation, XmHORIZONTAL,
	NULL);
    XtVaCreateManagedWidget(
	    "scale",
	    xmLabelGadgetClass,
	    parent,
	    XmNlabelString, scale,
	    XmNrecomputeSize, False,
	    NULL);
    /* make the text widgets */
    RCAS[zoomcount].wids.color_max = 
	XtVaCreateManagedWidget(
	    "color_max",
	    xmLabelGadgetClass,
	    parent,
	    XmNlabelString, s1,
	    XmNrecomputeSize, False,
	    NULL);


    DEBUG2("MCB: 2\n");
    /* now to make the color bar */
    XtVaGetValues(parent,
		  XmNdepth, &depth,
		  NULL);

    /* create a pixmap for the label widget */
    RCAS[zoomcount].wids.color_pix = 
	XCreatePixmap(canvas_display,
		      RootWindow(canvas_display, canvas_screen_num), /*screen*/
		      COLOR_BAR_WIDTH, /* width */
		      USER_COLORS,     /* height */
		      depth);         /* screen depth */
    RCAS[zoomcount].wids.color_bar = 
	XtVaCreateManagedWidget("colorbar",
				xmLabelWidgetClass,
				parent,
				XmNlabelType, XmPIXMAP,
				XmNlabelPixmap, RCAS[zoomcount].wids.color_pix,
				NULL);
	
   DEBUG2("MCB: 3\n");
    RCAS[zoomcount].wids.color_min = 
	XtVaCreateManagedWidget(
	    "color_min",
	    xmLabelGadgetClass,
	    parent,
	    XmNlabelString, s2,
	    XmNrecomputeSize, False,
	    NULL);

    XtVaCreateManagedWidget(
	"sep_1",
	xmSeparatorGadgetClass,
	parent,
	XmNorientation, XmHORIZONTAL,
	NULL);

    XtVaCreateManagedWidget(
	"min_blurb",
	xmLabelGadgetClass,
	parent,
	XmNlabelString, matmin,
	XmNrecomputeSize, False,
	NULL);
    RCAS[zoomcount].wids.input_min = 
	XtVaCreateManagedWidget(
	    "input_min",
	    xmLabelGadgetClass,
	    parent,
	    XmNlabelString, s2,
	    XmNrecomputeSize, False,
	    NULL);
    RCAS[zoomcount].wids.input_min_pos = 
	XtVaCreateManagedWidget(
	    "input_min_pos",
	    xmLabelGadgetClass,
	    parent,
	    XmNlabelString, s3,
	    XmNrecomputeSize, False,
	    NULL);
    XtVaCreateManagedWidget(
	"sep_2",
	xmSeparatorGadgetClass,
	parent,
	XmNorientation, XmHORIZONTAL,
	NULL);
    XtVaCreateManagedWidget(
	"max_blurb",
	xmLabelGadgetClass,
	parent,
	XmNlabelString, matmax,
	XmNrecomputeSize, False,
	NULL);
    RCAS[zoomcount].wids.input_max = 
	XtVaCreateManagedWidget(
	    "input_max",
	    xmLabelGadgetClass,
	    parent,
	    XmNlabelString, s1,
	    XmNrecomputeSize, False,
	    NULL);
    RCAS[zoomcount].wids.input_max_pos = 
	XtVaCreateManagedWidget(
	    "input_min_pos",
	    xmLabelGadgetClass,
	    parent,
	    XmNlabelString, s3,
	    XmNrecomputeSize, False,
	    NULL);
    XtVaCreateManagedWidget(
	"sep_3",
	xmSeparatorGadgetClass,
	parent,
	XmNorientation, XmHORIZONTAL,
	NULL);
    XmStringFree(scale);
    XmStringFree(s1);
    XmStringFree(s2);
    XmStringFree(s3);
    XmStringFree(matmin);
    XmStringFree(matmax);

}

/* fill in the colorbar */
void FillColorBar(Widget CB, int zoomcount)
{    
    float scale = 1.0;
    int idx,i, height, width;
    char debugstr[100];

    sprintf(debugstr,"FillColorBar called: zoomcount = %d\n",zoomcount);
    TRACE(debugstr);

    width = COLOR_BAR_WIDTH;
    height =USER_COLORS;

    if (height < USER_COLORS)
    {
	scale = USER_COLORS/height;
    }
    /* fill in the bottom  line */
    if (RCAS[zoomcount].force_black)
	idx = RCAS[zoomcount].what_is_black;
    else
	idx = RESERVED_COLORS;
    
    /* fprintf(stderr,"FillColorBar: idx = %d\n",idx); */

    XFillRectangle(canvas_display, RCAS[zoomcount].wids.color_pix, 
		   RCAS[zoomcount].gcs[idx],
		   0,0,width,height);

    for (i = 0; i < height-1; i++)
    {
	idx = RESERVED_COLORS+(((int) (USER_COLORS-(i*scale)))-1);
	/* XFR( display, window, GC, start_x, start_y, width, height) */
 	XFillRectangle(canvas_display, 
		       RCAS[zoomcount].wids.color_pix, 
		       RCAS[zoomcount].gcs[idx],
		       0,i,width,1);
    }
    TRACE("FillColorBar: exiting\n");
}

void kts_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
    /* kiss the sky call back */
    XtUnmanageChild(w);
}

void
DrawingAreaInput(Widget w, XtPointer client_data, XtPointer call_data)
{
    TRACE("DrawingAreaInput called\n");
}

/* #define MSG \
 * "Use the FileSelection dialog to find bitmap files to\n \
 * display in the scrolling area in the main window.  Use\n \
 * the edit menu to display the bitmap in different colors."
 */

#define MSG \
"MENU BAR OPTIONS:\n\n\
The File Menu:\n   \
To DESTROY a window, choose the 'Quit' option -- Quitting the\n   \
Main Window terminates emily.\n\
\n   \
A NEW MATRIX can be displayed by choosing the 'New' option on the Main\n   \
Window and use the dialog that pops up to specify a new file name.\n\
\n\
The Colors Menu:\n   \
Use this menu to CHOOSE A NEW COLORMAP -- this change affects ALL windows.\n\
\n\
The Background Menu:\n   \
Use this menu to CHOOSE A NEW BACKGROUND COLOR.  A new background color\n   \
affects only the current window and any of its children, created by zooming.\n\
\n\
The Scaling Menu:\n   \
Use this menu to CHANGE THE SCALING STRATEGY.  Currently, two types of\n   \
linear scaling and 2 types of logarithmic scaling are supported.\n\
\n\n\
OTHER OPERATIONS AND HINTS:\n\
The Zoom Icon:\n   \
Click on the Zoom Icon to START A ZOOM OPERATION.  After clicking on the\n   \
Zoom Icon, drag out the area of interest.  emily will CREATE A ZOOM\n   \
WINDOW for the selected submatrix.   Further zoom operations, changing\n   \
the background color, changing the colormap, and independently closing\n   \
the window are permitted operations on zoom windows.\n\
\n\
Miscellaneous:\n   \
Clicking on any point in the display section of any window will give \n   \
some information about that point. Changing either the Background or the\n   \
Scaling strategy can take a minute or so to be recomputed.\n "

void
help_cb(Widget widget, XtPointer client_data, XtPointer call_data)
{
    static Widget d1;
    
    if (!d1)
    {
	Arg args[1];
	XmString msg = XmStringCreateLtoR(MSG, XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[0], XmNmessageString, msg);
	d1 = XmCreateInformationDialog(RCAS[0].wids.toplevel, "help_dialog", args, 1);
    }
    XtManageChild(d1);
    XtPopup(XtParent(d1), XtGrabNone);
    
}


/* 
 * this function sets up the color bar
 * 
 * modified extensively, but on 7/27/94, STARTING_OUT_MAT_ADDR was added 
 */
void
SetColorMinMax(double min, double max, double inmin, double inmax, 
	       int minrow, int mincol, int maxrow, int maxcol, int zoomcount)
{
	XmString mins, maxs;
	XmString pos;
	char buffer[10];

	sprintf(buffer,"%9.1e",min);
	mins = XmStringCreateSimple(buffer);
	XtVaSetValues(RCAS[zoomcount].wids.color_min, 
		      XmNlabelString, mins, NULL);
	XmStringFree(mins);

	sprintf(buffer,"%9.1e",max);
	maxs = XmStringCreateSimple(buffer);

	XtVaSetValues(RCAS[zoomcount].wids.color_max, 
		      XmNlabelString, maxs, NULL);
	/* FillColorBar(RCAS[zoomcount].wids.color_bar, zoomcount);*/
	XmStringFree(maxs);

	sprintf(buffer,"%9.1e",inmin);
	mins = XmStringCreateSimple(buffer);
	XtVaSetValues(RCAS[zoomcount].wids.input_min, 
		      XmNlabelString, mins, NULL);
	XmStringFree(mins);

	sprintf(buffer,"%9.1e",inmax);
	maxs = XmStringCreateSimple(buffer);

	XtVaSetValues(RCAS[zoomcount].wids.input_max, 
		      XmNlabelString, maxs, NULL);

	sprintf(buffer,"(%6d,%6d)",minrow+STARTING_OUT_MAT_ADDR, 
		mincol+STARTING_OUT_MAT_ADDR);
	pos = XmStringCreateSimple(buffer);

	XtVaSetValues(RCAS[zoomcount].wids.input_min_pos, 
		      XmNlabelString, pos, NULL);

	sprintf(buffer,"(%6d,%6d)",maxrow+STARTING_OUT_MAT_ADDR, 
		maxcol+STARTING_OUT_MAT_ADDR);
	pos = XmStringCreateSimple(buffer);

	XtVaSetValues(RCAS[zoomcount].wids.input_max_pos, 
		      XmNlabelString, pos, NULL);


	FillColorBar(RCAS[zoomcount].wids.color_bar, zoomcount);
	XmStringFree(maxs);
}



/*
 * create menus 
 */
void
MakeMenus(Widget parent, int zoomcount) 
{

    Widget mb, menu, widget;
    char debugstr[100];
    char menu_name[100];
    XmString s1,s2,s3,s4,s5,s6,s7,s8,s9,s10,s11,s12,s13,s14,s15;

    sprintf(debugstr,"MakeMenus called: zoomcount = %d\n",zoomcount);
    TRACE(debugstr);
    sprintf(menu_name,"menu_bar%d",zoomcount);
    s1=XmStringCreateSimple("File");
    s2=XmStringCreateSimple("Colors");
    s3=XmStringCreateSimple("Background");
    s4=XmStringCreateSimple("Scaling");
    if (zoomcount == 0)
    {
	s5=XmStringCreateSimple("Help");
	mb = XmVaCreateSimpleMenuBar(parent, menu_name,
	   XmVaCASCADEBUTTON, s1, '\0',
	   XmVaCASCADEBUTTON, s2, '\0',
 	   XmVaCASCADEBUTTON, s3, '\0',
 	   XmVaCASCADEBUTTON, s4, '\0',
	   XmVaCASCADEBUTTON, s5, '\0',
           XmNtopAttachment, XmATTACH_FORM,
           XmNleftAttachment, XmATTACH_FORM,
           XmNrightAttachment, XmATTACH_FORM,
        				     
           NULL);
	XmStringFree(s5);
	/* tell menubar which button is the help menu */
	/* if (widget = XtNameToWidget(mb, "button_4"))
	 *{
	 *   XtVaSetValues(mb,
	 *		  XmNmenuHelpWidget, widget,
	 *		  NULL);
	 * }
	 */
    }
    else
    {
	mb = XmVaCreateSimpleMenuBar(parent, menu_name,
	   XmVaCASCADEBUTTON, s1, '\0',
	   XmVaCASCADEBUTTON, s2, '\0',
 	   XmVaCASCADEBUTTON, s3, '\0',
 	   XmVaCASCADEBUTTON, s4, '\0',
           XmNtopAttachment, XmATTACH_FORM,
           XmNleftAttachment, XmATTACH_FORM,
           XmNrightAttachment, XmATTACH_FORM,
           NULL);
    }
    XmStringFree(s1);
    XmStringFree(s2);
    XmStringFree(s3);
    XmStringFree(s4);

	
    sprintf(menu_name,"file_menu%d",zoomcount);
    s4=XmStringCreateSimple("Quit");
    s5=XmStringCreateSimple("Print");
    if (zoomcount == 0)
    {
	s1=XmStringCreateSimple("Clear");
	s2=XmStringCreateSimple("New");
	s3=XmStringCreateSimple("Refresh");
	XmVaCreateSimplePulldownMenu(mb, menu_name, 0, fileMenuCallback,
      	  XmVaPUSHBUTTON, s1, 'C', NULL, NULL,
	  XmVaPUSHBUTTON, s2, 'N', NULL, NULL,
	  XmVaPUSHBUTTON, s3, 'R', NULL, NULL,
	  XmVaPUSHBUTTON, s5, 'P', NULL, NULL,
	  XmVaPUSHBUTTON, s4, 'Q', NULL, NULL,
          XmNcolormap, Default_cmap,
          XmNdepth, canvas_depth,
          XmNvisual, canvas_visual,
	  NULL);
	XmStringFree(s1);
	XmStringFree(s2);
	XmStringFree(s3);
    }
    else
    {
	XmVaCreateSimplePulldownMenu(mb, menu_name, 0, fileMenuCallback,
	  XmVaPUSHBUTTON, s5, 'P', NULL, NULL,
	  XmVaPUSHBUTTON, s4, 'Q', NULL, NULL,
          XmNcolormap, Default_cmap,
          XmNdepth, canvas_depth,
          XmNvisual, canvas_visual,
	  NULL);
    }
    XmStringFree(s4);
    XmStringFree(s5);

    sprintf(menu_name,"color_menu%d",zoomcount);

    s1=XmStringCreateSimple("Grayscale");
    s2=XmStringCreateSimple("Spectral");
    s3=XmStringCreateSimple("Ware's");
    s4=XmStringCreateSimple("Hot");
    s5=XmStringCreateSimple("Cool");
    s6=XmStringCreateSimple("Copper");
    s7=XmStringCreateSimple("Sunrise");
    s8=XmStringCreateSimple("Sunset");
    s9=XmStringCreateSimple("Weather");  
    /* s9=XmStringCreateSimple("Red-Green"); */
    /* s10=XmStringCreateSimple("Spect2"); */
    s10=XmStringCreateSimple("Robertson's");
    menu = XmVaCreateSimplePulldownMenu(mb, menu_name, 1, ColorMapCallback,
          XmVaRADIOBUTTON, s1, 'G', NULL, NULL,
          XmVaRADIOBUTTON, s2, 'S', NULL, NULL,
          XmVaRADIOBUTTON, s3, 'P', NULL, NULL,
          XmVaRADIOBUTTON, s4, 'H', NULL, NULL,
          XmVaRADIOBUTTON, s5, 'C', NULL, NULL,
          XmVaRADIOBUTTON, s6, 'o', NULL, NULL,
          XmVaRADIOBUTTON, s7, 'u', NULL, NULL,
          XmVaRADIOBUTTON, s8, 'n', NULL, NULL,
       /* XmVaRADIOBUTTON, s9, 'R', NULL, NULL, */
          XmVaRADIOBUTTON, s9, 'W', NULL, NULL,
     /*   XmVaRADIOBUTTON, s10, '2', NULL, NULL, */
          XmVaRADIOBUTTON, s10, 'B', NULL, NULL,
          XmNcolormap, Default_cmap,
          XmNdepth, canvas_depth,
          XmNvisual, canvas_visual,
          XmNradioBehavior, True,
          XmNradioAlwaysOne, True,
          NULL);

    XmStringFree(s1);
    XmStringFree(s2);
    XmStringFree(s3);
    XmStringFree(s4);
    XmStringFree(s5);
    XmStringFree(s6);
    XmStringFree(s7);
    XmStringFree(s8);
    XmStringFree(s9);
    XmStringFree(s10);
    /* XmStringFree(s11); */
    /* get current colormap */
    sprintf(menu_name,"button_%d",RCAS[zoomcount].colormap);
    if (widget = XtNameToWidget(menu,menu_name))
	XtVaSetValues(widget,
		      XmNset, True,
		      NULL);
    
    sprintf(menu_name,"bcolor_menu%d",zoomcount);

    s1=XmStringCreateSimple("Natural");
    s2=XmStringCreateSimple("Black");
    s3=XmStringCreateSimple("White");
    s4=XmStringCreateSimple("Gray");
    s5=XmStringCreateSimple("Red");
    s6=XmStringCreateSimple("Green");
    s7=XmStringCreateSimple("Blue");
    menu = XmVaCreateSimplePulldownMenu(mb, menu_name, 2, BackgroundColorCallback,
	  XmVaRADIOBUTTON, s1, 'N', NULL, NULL,
          XmVaRADIOBUTTON, s2, 'B', NULL, NULL,
          XmVaRADIOBUTTON, s3, 'W', NULL, NULL,
          XmVaRADIOBUTTON, s4, 'G', NULL, NULL,
          XmVaRADIOBUTTON, s5, 'R', NULL, NULL,
          XmVaRADIOBUTTON, s6, 'e', NULL, NULL,
          XmVaRADIOBUTTON, s7, 'l', NULL, NULL,
          XmNcolormap, Default_cmap,
          XmNdepth, canvas_depth,
          XmNvisual, canvas_visual,
          XmNradioBehavior, True,
          XmNradioAlwaysOne, True,
          NULL);

    XmStringFree(s1);
    XmStringFree(s2);
    XmStringFree(s3);
    XmStringFree(s4);
    XmStringFree(s5);
    XmStringFree(s6);
    XmStringFree(s7);

    /* figure out current background color */
    if (RCAS[zoomcount].force_black)
	sprintf(menu_name,"button_%d",RCAS[zoomcount].what_is_black+1);
    else
	strcpy(menu_name,"button_0");

    /* light up the current background color */
    if (widget = XtNameToWidget(menu,menu_name))
	XtVaSetValues(widget,
		      XmNset, True,
		      NULL);

    /* scale menu stuff */
    sprintf(menu_name,"scale_menu%d",zoomcount);
    s1=XmStringCreateSimple("Linear");
    s2=XmStringCreateSimple("Absolute Linear");
    s3=XmStringCreateSimple("Abs. Natural Log");
    s4=XmStringCreateSimple("Abs. Log Base 10");
    menu = XmVaCreateSimplePulldownMenu(mb, menu_name, 3, ScaleMenuCallback,
      	  XmVaRADIOBUTTON, s1, 'L', NULL, NULL,
	  XmVaRADIOBUTTON, s2, 'A', NULL, NULL,
	  XmVaRADIOBUTTON, s3, 'N', NULL, NULL,
	  XmVaRADIOBUTTON, s4, '1', NULL, NULL,
          XmNcolormap, Default_cmap,
          XmNdepth, canvas_depth,
          XmNvisual, canvas_visual,
          XmNradioBehavior, True,
          XmNradioAlwaysOne, True,				
	  NULL);

    XmStringFree(s1);
    XmStringFree(s2);
    XmStringFree(s3);
    /* set default scale to linear */
    if (RCAS[zoomcount].sstrat == SClog)
    {	
	if (RCAS[zoomcount].log10 != 0)
	{
	    sprintf(menu_name,"button_3");
	}
	else
	{
	    sprintf(menu_name,"button_2");
	}
    }
    else if (RCAS[zoomcount].sstrat == SCabslinear)
	sprintf(menu_name,"button_1");
    else /* default is SClinear2 -- linear scaling */
    {
	sprintf(menu_name,"button_0");
    }

    /* light up the current scaling strategy */
    if (widget = XtNameToWidget(menu,menu_name))
	XtVaSetValues(widget,
		      XmNset, True,
		      NULL);

    XmVaCreateSimplePulldownMenu( mb,
				  "help_menu", 4,
				  help_cb,
				  XmNcolormap, Default_cmap,
				  XmNdepth, canvas_depth,
				  XmNvisual, canvas_visual,
	         XmVaPUSHBUTTON, XmStringCreateSimple("Help"), 'H',NULL, NULL,
				  NULL);

    XtManageChild(mb);
    RCAS[zoomcount].wids.menu = mb;
    TRACE("MakeMenus done!\n");
}
/*
 * 0 gray
 * 1 spectral
 * 2 pete's
 * 3 hot
 * 4 cool
 * 5 copper
 * 6 sunrise
 * 7 sunset
 * 8 weather
 */
void
ColorMapCallback(Widget w, XtPointer client_data, XtPointer call_data) {

    int item_no = (int) client_data;
    int i;

    GetMapping(item_no, Default_cmap);
    /* for all windows */
    for (i = 0; i < MAXZOOM; i++)
    {
	/* if the window is being used */
	if (RCAS[i].used)
	{
	    /* set the colormap to the current one */
	    RCAS[i].colormap = item_no;
	    /* and fill in the color bar */
	    FillColorBar(RCAS[i].wids.color_bar, i); 
	}
    }
}

/*
 * 0 natural
 * 1 black
 * 2 white
 * 3 gray
 * 4 red
 * 5 green
 * 6 blue
 */
void
BackgroundColorCallback(Widget w, XtPointer client_data, XtPointer call_data) {

    int item_no = (int) client_data;
    int zc;
    char debugstr[150];

    TRACE("BackgroundColorCallback: entering\n");
    zc = GetZoomcount(GETMENUBAR(w));

    sprintf(debugstr,
	    "Background Color Callback: zc = %d, w = %d, p^3 = %d, RCAS[0].wids.menu = %d\n",
	    zc,w,GETMENUBAR(w), RCAS[0].wids.menu);
    DEBUG2(debugstr);

    if (item_no == 0)
    {
	RCAS[zc].force_black = 0;
    }
    else
    {
	RCAS[zc].force_black = 1;
	RCAS[zc].what_is_black = item_no - 1;
    }
    RCAS[zc].action = RCASscale;
    if (ReadCompressAndScaleMat( &RCAS[zc] ) == OK)
    {
	sprintf(debugstr,"BackgroundColorCallback: RCAS OK: copy pix\n");
	DEBUG2(debugstr);
	/*
	 * CopyPixToCanvas( canvas_display, RCAS[zc].pix, 
	 *		 RCAS[zc].window,
	 *		 RCAS[zc].screen_height, RCAS[zc].screen_width, RCAS[zc].gcs[7] );
	 */
	ShowRCASResults(&RCAS[zc]);
    }
    TRACE("BackgroundColorCallback: leaving\n");
}

/* 
 * 0 = linear
 * 1 = abs linear
 * 2 = logarithmic
 * 3 = log 10
 */
void
ScaleMenuCallback( Widget w, XtPointer client_data, XtPointer call_data)
{
    int item_no = (int) client_data;
    int zc;
    char debugstr[150];
    zc = GetZoomcount(GETMENUBAR(w));

    if ((item_no == SCALE_LN) || (item_no == SCALE_LOG_10)) /* logarithmic */
    {
	/* kinda hacky, but ... */
	RCAS[zc].log10 = (item_no == SCALE_LOG_10); 
	/* performance hack if log scaling already done, don't redo it,
	   just call ShowRCASResults to update the screen... */
	if (RCAS[zc].sstrat == SClog)
	{
	    ShowRCASResults(&RCAS[zc]);
	    return;
	}
	else
	    RCAS[zc].sstrat = SClog;
    }
    else if (item_no == SCALE_ABS_LINEAR)
	RCAS[zc].sstrat = SCabslinear;
    else /* 0 and default */
	RCAS[zc].sstrat = SClinear2;

    /* now try the new scaling */
    RCAS[zc].action = RCASscale;

    if (ReadCompressAndScaleMat( &RCAS[zc] ) == OK)
    {
	/* CanvasClearWindow(zc); */
	ShowRCASResults( &RCAS[zc] );
    }
}
/*
 * 0 = clear
 * 1 = new/print (zoom window)
 * 2 = refresh
 * 3 = print (main window)
 * 4 = quit
 * 
 * Modified 
 */
void
fileMenuCallback(Widget w, XtPointer client_data, XtPointer call_data) {

    static Widget dialog; /* static for reuse */
    int item_no = (int) client_data;
    extern void CanvasClearWindow();
    extern void load_pixmap();
    int zc;
    char debugstr[100];


    /* assume that the widget is a direct kiddo of the menubar -- 3/30 */
    zc = GetZoomcount(GETMENUBAR(w));
    sprintf(debugstr,"fileMenuCallback: zc = %d\n",zc);
    TRACE(debugstr);

    if (zc < 0) /* assume the file menu window -- 3/26/94 */
	zc = 0; 
    /* for all other windows than the main window, two options on
       the file menu -- Print and QUIT (after 1/13/95) */
    if (zc > 0)
    {
	if (item_no == 0) /* print */
	{
	    PrintDialog(RCAS[zc].wids.toplevel,zc);
	    return;
	}
	else if (item_no == 1) /* quit */
	{
	    CloseZoomWindow(zc); /* call put in on 4/14/94 */
	    return;
	}
	else /* what? */
	    exit(BAD);
    }
    /* if here, on main window */
    switch (item_no)
    {
    case 0: /* clear */
    
/*	CanvasClearWindow(zc); */
	XClearWindow(XtDisplay(RCAS[zc].wids.canvas), 
		     XtWindow(RCAS[zc].wids.canvas)); 
	break;
    case 2: /* refresh */
	/* CopyPixToCanvas( 
	    RCAS[zc].display, 
	    RCAS[zc].pix, 
	    RCAS[zc].window, 
	    RCAS[zc].screen_height,
	    RCAS[zc].screen_width,
	    RCAS[zc].gcs[0]);
	     */
	ShowRCASResults(&(RCAS[zc]));
	break;
    case 3: /* print */
	PrintDialog(RCAS[zc].wids.toplevel,zc);
	break;
    case 4: /* exit */
	exit(0);
    case 1: /* new file */
    /* new was selected.  create a file selection dialog with callback */
	if (!dialog)
	{
	    DEBUG3("FileMenuCallback: creating file selection dialog\n");
	    dialog = XmCreateFileSelectionDialog(
		RCAS[0].wids.toplevel, "file_sel", NULL, 0);
	    DEBUG3("FileMenuCallback: adding ok callback\n");
	    XtAddCallback(dialog, XmNokCallback, load_pixmap, NULL);
	    /* HMMM!!! */
	    DEBUG3("FileMenuCallback: adding cancel callback \n");
	    XtAddCallback(dialog, XmNcancelCallback, kts_cb, NULL); 
	}
	DEBUG2("FileMenuCallback: managing dialog\n");
	XtManageChild(dialog);
	DEBUG2("FileMenuCallback: calling XtPopup\n");
	XtPopup(XtParent(dialog), XtGrabNone);
	break;
    default:
	break;
    }
    return;
}

/* for load_pixmap() callback:
 * The ok button was selected from file selection dialog.  Try to read
 *  file as bitmap.
 */
    
/* void load_pixmap(Widget dialog, XtPointer client_data, XmFileSelectionBoxCallbackStruct *cbs) */
void load_pixmap(Widget dialog, XtPointer client_data, XtPointer call_data)
{

    static char readfile[200], savefile[200];
    int fn_saved = 0;
    char debugstr[100];
    Pixmap pixmap;
    DaMatrix D;
    DenseMatrix comp;
    char *file;
    int zc = 0; /* FORCE TO MAIN WINDOW */
    XmFileSelectionBoxCallbackStruct *cbs; /* added 7/28 */

    /* 7/28 -- move this off arg list to prevent compiler warnings */
    cbs = (XmFileSelectionBoxCallbackStruct *) call_data;
    
    if (cbs) /* NULL iff called from change_color() */
    {
	if (!XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &file))
	    return; /* internal error */
	strcpy(readfile, file);
	XtFree(file); /* free allocated data from the string */
    }


    RCAS[zc].action = RCASread;
    if (RCAS[zc].file != (char *) NULL)
    {
	fn_saved = 1;
	sprintf(debugstr,"load_pixmap: copying %s\n",RCAS[zc].file);
	DEBUG2(debugstr);
	/* fprintf(stderr,"%s",debugstr); */
	strcpy(savefile, RCAS[zc].file);
    }

    RCAS[zc].file = readfile;
    if (RCAS[zc].input->type == Dense) /* currently a test matrix */
    {
	FreeDenseMatrix(RCAS[zc].input->data.d);
	RCAS[zc].input->type = HB;
	
	if ((RCAS[zc].input->data.hb = (HBMatrix *) MALLOC(1,sizeof(HBMatrix))) == (HBMatrix *) NULL)
	{
#ifdef STANDALONE
	    error("load pixmap couldn't get a matrix!\n",MEMERR);
#else
	    fprintf(stderr,"load_pixmap couldn't get a matrix\n");
#endif
	    return;
	    /* shouldn't get here */
	}
    }
	
    if (ReadCompressAndScaleMat( &RCAS[zc] ) == OK)
    {
	/* CanvasClearWindow(zc); */
	ShowRCASResults( &RCAS[zc] );
    }
    else if (fn_saved) /* restore old file name, if we saved it */
    {
	sprintf(debugstr,"load_pixmap: replacing file\n");
	DEBUG2(debugstr);
	RCAS[zc].file = savefile;
    }
    else
    {
	RCAS[zc].file = (char *) NULL;
    }
    /* TBD: should check to see if there's a cancel callback */
    XtCallCallbacks(dialog,XmNcancelCallback,NULL);
    /* XtUnmanageChild(dialog); */
}


/*
 * There is no widget class that conveniently handles the function of the
 * icons, so we make them out of a Label widget and a Frame widget.  The
 * Label widget does not normally handle events, so we have to add an action
 * to the action list.  After adding the action, we need a translation to
 * call the action.  Since translations can be specified in the resource
 * file we put the translation table there for easy debugging.
 *
 * When we get a button click, we then reverse the shadow on the Frame widget
 * so it looks inset, then we also change the background color of the interior
 * Label widget so it is slightly darker to enhance the 3-D look.
 * 
 * Lastly, we need to cancel the previous mode by calling
 * the function with 0.  If the previous mode is the same as the depressed
 * mode, then we unset it, and no mode is set.
 */


static Icon *icons;		/* Hold pointers to the icons. */

void
MakeIcons(Widget parent,int zoomcount) {	
    static XtActionsRec actions[] = {{"Click", click}};

	/*
	 * Install 'Click()' as an action so we can get input events from
	 * the Label widget.
	 */

    XtAppAddActions(AppCon, actions, XtNumber(actions));

	/*
	 * Create a RowColumn widget to hold the icons.
	 */

    RCAS[zoomcount].wids.icons = XtVaCreateManagedWidget(
	"icon_box_row_column",
	xmRowColumnWidgetClass,
	parent,
	XmNorientation, XmVERTICAL,
	XmNnumColumns, 1,
	XmNpacking, XmPACK_COLUMN,
	XmNadjustLast, False,
	NULL);

	/* InitializeModes(); */

	/* return rc; */
}

void
MakeIcon(void (*f)(), char *file, int zoomcount) {
    Icon *ic = (Icon *) malloc(sizeof(Icon));
    Colormap cmap;
    Pixel fgc, tsc, bsc;
    char name[100];
    String trans_string = "<Btn1Down>: Click()";
    XtTranslations t;
    static int i;

    /*
     * Create the Frame widget to give the 3-D appearance.
     */

    sprintf(name, "frame%d", i);
    ic->frameWidget = XtVaCreateManagedWidget(
	name,                        /* name of the widget */
	xmFrameWidgetClass,          /* widget class */
	RCAS[zoomcount].wids.icons,	/* parent widget */
	XmNshadowType, XmSHADOW_OUT, /* shadow type */
	NULL);

    /*
     * Create the Label widget to hold the Pixmap (gotten from the resource
     * file).
     */

    sprintf(name, "prim%d", i);
    ic->labelWidget = XtVaCreateManagedWidget(
	name,			/* widget name */
	xmLabelWidgetClass,		/* widget class */
	ic->frameWidget,		/* parent widget */
	NULL);

    t = XtParseTranslationTable(trans_string);
    XtOverrideTranslations(ic->labelWidget, t);

    /*
     * Now we need to calculate the correct color that the label should hvae
     * when it's been depressed.  We call XmGetColors() to calculate this
     * for us.
     */

    /*
     * First get the colormap and get the background color.  XmGetColors()
     * uses the background color to calculate the other colors.
     */

    XtVaGetValues(ic->labelWidget,
		  XmNcolormap, &cmap,
		  XmNbackground, &ic->normalColor,
		  NULL);

    /*
     * Calculate the colors.  This function calculates other colors too, but
     * we are interested only in the "select" color.
     */

    XmGetColors(XtScreen(ic->labelWidget), cmap,
		ic->normalColor, &fgc, &tsc, &bsc, &ic->selectColor);

    /*
     * Now we need to load in the bitmap for the button.  We create two
     * Pixmaps for this.  One has the outset color as the background, the
     * other has the inset color for the background.
     */

    /* 4/14/94 -- load pixmap from internal data vs. from a file so
     *   executable can be executed from any directory.
     */

    /*
     * ic->normalPixmap = XmGetPixmap(XtScreen(ic->labelWidget), file,
     *                              fgc, ic->normalColor);
     * ic->selectPixmap = XmGetPixmap(XtScreen(ic->labelWidget), file,
     *			            fgc, ic->selectColor);
     */

    /* this blows up now, need to repl. XtWindow() with something else
       or install an image.  How is XmGetPixmap getting away with just a
       screen? */

    ic->normalPixmap = XCreatePixmapFromBitmapData(canvas_display,
			      RootWindowOfScreen(XtScreen(ic->labelWidget)),
			      ZoOm_BiTs,
			      ZoOm_WiDtH,ZoOm_HeIgHt,
			      fgc,ic->normalColor,
			      canvas_depth);
    ic->selectPixmap = XCreatePixmapFromBitmapData(canvas_display,
			      RootWindowOfScreen(XtScreen(ic->labelWidget)),
			      ZoOm_BiTs,
			      ZoOm_WiDtH,ZoOm_HeIgHt,
			      fgc,ic->selectColor,canvas_depth);
						   
    /*
     * Set the label to display the Pixmap.
     */

    XtVaSetValues(ic->labelWidget,
		  XmNlabelType, XmPIXMAP,
		  XmNlabelPixmap, ic->normalPixmap,
		  NULL);

    /*
     * Set the function to call to activate the mode represented by this
     * icon.
     */

    ic->function = f;

    ic->suc = icons;
    icons = ic;
    i++;
}

/*
 * This is the action function that is called when we get a button 1 down
 * event.  In here, we need to change the appearance of any buttons that
 * change state, and call the activation function.
 */

/* TJL: 3/25/94 -- this prob. won't work but I'm leaving it alone for now */
void
click(Widget w, XEvent *e, String *p,  Cardinal *np) {
	static Icon *set = 0;
	Icon *ic;
	
	if (set != 0) 
	{		
	    /* set == -1 means no mode set */
	    
	    /* Change Pixmap. */
	    XtVaSetValues(set->labelWidget,
			  XmNbackground, set->normalColor,
			  XmNlabelPixmap, set->normalPixmap,
			  NULL);
	    /* Change shadow to outset */
	    XtVaSetValues(set->frameWidget,
			  XmNshadowType, XmSHADOW_OUT,
			  NULL);
	    /* Call function to deactivate the mode. */
	    (*set->function)(0);
	}
	
	/* If we have clicked on a button that is not now depressed, then 
	   change the appearance of that button, and activate that mode. */
	if (set == 0 || (w != set->labelWidget)) 
	{
	    for (ic = icons; ic != NULL; ic = ic->suc) 
	    {
		if (w == ic->labelWidget) 
		{
		    set = ic;
		    XtVaSetValues(ic->labelWidget,
				  XmNbackground, ic->selectColor,
				  XmNlabelPixmap, ic->selectPixmap,
				  NULL);
		    XtVaSetValues(ic->frameWidget,
				  XmNshadowType, XmSHADOW_IN,
				 NULL);
		    (*ic->function)(1);
		    break;
		}
	    }
	    /* TJL: if at end of list and no icon was found to set,
	       assume not set and set "set" to NULL.  This allows
	       click() to be called by zoom code to release the button
	       even though the button wasn't actually clicked up */
	    if (ic == NULL)
		set = 0;
	} 
	else 
	{
	    set = 0;
	}
}

void GetMapping(int type, Colormap c )
{
    unsigned long Plane_masks[1];
    int ncolors=MAXIMUM_COLORS,i, status;
    RGB colors[MAXIMUM_COLORS];

    /* init colors array */
    TRACE("GetMapping: entering\n");
    for (i = 0; i < MAXIMUM_COLORS; i++)
	colors[i].Red = colors[i].Green = colors[i].Blue = 0;
    

    /* get the color cells -- this is a fatal error if can't be done */

    if (ALLOCATED == False)
    {
	status = XAllocColorCells(canvas_display,c,False,
				  Plane_masks,0,allocated_colors,ncolors);
	if (status == 0)
	{
	    fprintf(stderr,"Didn't get the color cells!\n");
	    exit(BAD);
	}
	/* printf("GM: got cells\n");*/
    }
    GetColors(type, colors);
    /* printf("GM: GotColors, going for  LoadColorTable\n");*/

    LoadColorTable(colors,c);
    TRACE("GetMapping: see ya!\n");
}

void LoadColorTable(RGB colors[MAXIMUM_COLORS], Colormap c) 
{
    int i;

    TRACE("LoadColorTable entered\n");
    for (i=0;i<MAXIMUM_COLORS;i++) 
    {
	new_colors[i].pixel = allocated_colors[i];
	/* Range checking */

	colors[i].Red = (((colors[i].Red) >255) ?  (printf("Red component of Color[%d] out of range.\n",i), 255) :  (colors[i].Red));

	colors[i].Red = (((colors[i].Red) <0) ?  (printf("Red component of Color[%d] out of range.\n",i),0) :  (colors[i].Red));
	
	colors[i].Green = (((colors[i].Green) >255) ?  (printf("Green component of Color[%d] out of range.\n",i),255) : (colors[i].Green));
	
	colors[i].Green = (((colors[i].Green) <0) ?  (printf("Green component of Color[%d] out of range.\n",i),0) :  (colors[i].Green));

	colors[i].Blue = (((colors[i].Blue) >255) ? (printf("Blue component of Color[%d] out of range.\n",i),255) : (colors[i].Blue));

	colors[i].Blue = (((colors[i].Blue) <0) ?   (printf("Blue component of Color[%d] out of range.\n",i),0) : (colors[i].Blue));

	new_colors[i].red = 256*colors[i].Red;
	new_colors[i].green = 256*colors[i].Green;
	new_colors[i].blue = 256*colors[i].Blue;
	new_colors[i].flags = DoRed | DoGreen | DoBlue;
	
	/* debugging */
	
	/* printf("color[%d]: maps to %d as: %d %d %d\n",i,new_colors[i].pixel,
         * colors[i].Red, colors[i].Green,colors[i].Blue); 
	 */

       } 
    /* store the colors */
    XStoreColors(canvas_display,c,new_colors,MAXIMUM_COLORS);

    ALLOCATED = True;
    TRACE("LoadColorTable: done\n");

}

 void getGC(Display *display, Window win, GC *gc_ptr, unsigned long col)
{
    unsigned long valuemask = 0;
    XGCValues values;
    unsigned int line_width = 1;
    int line_style = LineSolid;
    int cap_style = CapRound;
    int join_style = JoinRound;

    *gc_ptr = XCreateGC(display, win, valuemask, &values);
    
    /* UNCOMMENT IF I WANT TO HAVE THE GC'S WRITE TEXT */
    /* XSetFont(display, *gc_ptr, font_info->fid);*/
    XSetFillRule(display, *gc_ptr, WindingRule);
    XSetFillRule(display, *gc_ptr, FillSolid);
    XSetForeground(display, *gc_ptr, col);

    XSetLineAttributes(display, *gc_ptr, line_width, line_style, 
                       cap_style, join_style);
}

void GetColors(int type, RGB *colors)
{
    s_color *s;
    int i = 0;
    int cur_color, max_color;
    RGB *cur, *next;
    int portion;
    float fportion;
    int j;
    float cur_part, next_part;

    switch (type)
    {
    case WEATHER:
	s = weather_values;
	break;
    case SUNSET:
	s = sunset_values;
	break;
    case SUNRISE:
	s = sunrise_values;
	break;
    case COPPER:
	s = copper_values;
	break;
    case COOL:
	s = cool_values;
	break;
    case HOT:
	s = hot_values;
	break;
    case SPECTRAL:
	s = spectral_values;
	break;
    case PETE:
	s = pete_values;
	break;

   /* case SPECTRAL2: */
  /* s = spectral_values2;*/
	/* 	break;*/
    case BLUE_YELLOW:
	s = blue_yellow_values;
	break;
    case GRAYSCALE:
    default:
	s = gray_values;
	break;
    }

    
    /* add a few, even if duplicates */
    /* assure we have black, white, gray, red, green, and blue */

    /* NOTE: these are the RESERVED_COLORS additional colors */

    /* black */
    colors[BLACK_INDEX].Red =  0;
    colors[BLACK_INDEX].Green =  0;
    colors[BLACK_INDEX].Blue =  0;

    /* white */
    colors[WHITE_INDEX].Red =  255;
    colors[WHITE_INDEX].Green =  255;
    colors[WHITE_INDEX].Blue =  255;

    /* gray */
    colors[GRAY_INDEX].Red =  128;
    colors[GRAY_INDEX].Green =  128;
    colors[GRAY_INDEX].Blue =  128;

    /* red */
    colors[RED_INDEX].Red =  255;
    colors[RED_INDEX].Green =  0;
    colors[RED_INDEX].Blue =  0;

    /* green */
    colors[GREEN_INDEX].Red =  0;
    colors[GREEN_INDEX].Green = 255;
    colors[GREEN_INDEX].Blue =  0;

    /* blue */
    colors[BLUE_INDEX].Red =  0;
    colors[BLUE_INDEX].Green =  0;
    colors[BLUE_INDEX].Blue =  255;

    /* foreground */
    colors[FG_INDEX].Red = (FGColor.red >> 8);
    colors[FG_INDEX].Green = (FGColor.green >> 8);
    colors[FG_INDEX].Blue = (FGColor.blue >> 8);

    /* background */
    colors[BG_INDEX].Red = (BGColor.red >> 8);
    colors[BG_INDEX].Green = (BGColor.green >> 8);
    colors[BG_INDEX].Blue = (BGColor.blue >> 8);

    cur_color = RESERVED_COLORS;
    while (s[i].portion > 0)
    {
	cur = &(s[i].value);
	next = &(s[i+1].value);
	portion = s[i].portion;
	fportion = (float) portion;
	i++;
	
	for (j = 0; j < portion; j++)
	{
	    cur_part = (fportion - j) / fportion;
	    next_part = ((float) j) / fportion;
	    colors[cur_color].Red =  
		(int) (cur_part * cur->Red + next_part * next->Red);
	    colors[cur_color].Green =  
		(int) (cur_part * cur->Green + next_part * next->Green);
	    colors[cur_color].Blue =  
		(int) (cur_part * cur->Blue + next_part * next->Blue);
/*	    printf("%d: Red = %d, Green = %d, Blue = %d\n",cur_color,
		   colors[cur_color].Red,colors[cur_color].Green,colors[cur_color].Blue); */
	    cur_color++;
	}
    } /* end of while */
} /* end of GetColors */

void CopyPixToCanvas( Display *display, Pixmap pix, Window w,
		 Dimension canvas_height, Dimension canvas_width, GC gc )
{
    int src_x = 0; /* start of src x */
    int src_y = 0; /* start of src y */ 
    int dest_x = 0; /* start of dest x */
    int dest_y = 0; /* start of dest y */
    int i, j; /* for debugging */
    char debugmsg[100];

    TRACE("CopyPixToCanvas entered\n");
    XCopyArea( display, 
	       pix,   /* src */
	       w,  /* dest */
	       gc,  /* gc */
	       src_x, src_y, 
	       (unsigned int) canvas_width, /* <-- width to be copied */
	       (unsigned int) canvas_height, /* <--- height to be copied */
	       dest_x, dest_y);

    XFlush(display);
    TRACE("CopyPixToCanvas done\n");
}

void
InitializeModes(int zoomcount) 
{
/*    MakeIcon(HBZoomHookup, "zoom.xbm", zoomcount);  */
    MakeIcon(HBZoomHookup, "zoom", zoomcount);

}

/* 
 * 1/14/95 -- code pulled out of ShowRCASResults so it can be used
 *            for print routines.  Think of it as a macro expansion --
 *            not designed for human use... :-(
 * 
 * by tom loos
 */
void GetMaxMinEtc(RCASParameters *InRCAS, double *min, double *max, 
		  double *in_min, double *in_max, int *minrow, int *mincol, 
		  int *maxrow,int *maxcol, 
		  int *submat_minrow, int *submat_mincol,
		  int *submat_maxrow, int *submat_maxcol, 
		  int *num_rows, int *num_cols, char **key)
{
    if (InRCAS->sstrat == SCabslinear)
    {
	*min = InRCAS->absmin;
	*max = InRCAS->absmax;
	*in_min = InRCAS->input_absmin;
	*in_max = InRCAS->input_absmax;
	*minrow = InRCAS->input_absmin_row;
	*mincol = InRCAS->input_absmin_col;
	*maxrow = InRCAS->input_absmax_row;
	*maxcol = InRCAS->input_absmax_col;
    }
    else if (InRCAS->sstrat == SClog)
    {
	*min = InRCAS->absmin;
	*max = InRCAS->absmax;
	*in_min = InRCAS->input_absmin;
	*in_max = InRCAS->input_absmax;

	if (*min == 0.0)
	    *min = log(10) * -MAX_SCALING_EXPONENT;
	else
	    *min = log(*min);

	/* UNCOMMENT IF YOU WANT INPUT TO SHOW LOG VALS, TOO */
	/*
	 * if (in_min == 0.0)
	 *   in_min = log(10) * -MAX_SCALING_EXPONENT;
	 * else
	 *   in_min = log(in_min);
	 */

	if (*max == 0.0)
	    *max = log(10) * -MAX_SCALING_EXPONENT;
	else
	    *max = log(*max);

	/* UNCOMMENT IF YOU WANT INPUT TO SHOW LOG VALS, TOO */
	/*
	 * if (in_max == 0.0)
	 *   in_max = log(10) * -MAX_SCALING_EXPONENT;
	 *else
	 *    in_max = log(in_max);
	 */
	if (InRCAS->log10 != 0)
	{
	    *in_min *= ONE_OVER_LN_10;
	    *in_max *= ONE_OVER_LN_10;
	    *max *= ONE_OVER_LN_10;
	    *min *= ONE_OVER_LN_10;
	}
	*minrow = InRCAS->input_absmin_row;
	*mincol = InRCAS->input_absmin_col;
	*maxrow = InRCAS->input_absmax_row;
	*maxcol = InRCAS->input_absmax_col;
    }
    else /* scale strat is SClinear2 -- linear scaling */
    {
	*min = InRCAS->min;
	*max = InRCAS->max;
	*in_min = InRCAS->input_min;
	*in_max = InRCAS->input_max;
	*minrow = InRCAS->input_min_row;
	*mincol = InRCAS->input_min_col;
	*maxrow = InRCAS->input_max_row;
	*maxcol = InRCAS->input_max_col;
    }
    
    *key = InRCAS->input->hb_head.Key;
    *submat_minrow = InRCAS->input_startrow+STARTING_OUT_MAT_ADDR;
    *submat_mincol = InRCAS->input_startcol+STARTING_OUT_MAT_ADDR;

    *submat_maxrow = InRCAS->input_endrow+STARTING_OUT_MAT_ADDR;
    *submat_maxcol = InRCAS->input_endcol+STARTING_OUT_MAT_ADDR;

    if (InRCAS->input->type == HB)
    {
	*num_rows = InRCAS->input->hb_head.NumRows;
	*num_cols = InRCAS->input->hb_head.NumCols;
    }
    else
    {
	*num_rows = InRCAS->input->d_NumRows;
	*num_cols = InRCAS->input->d_NumCols;
    }

}    
/*
 * This function is the primary display function for emily.
 * 
 * It's been hacked a lot, but on  7/27/94 STARTING_OUT_MAT_ADDR was added.
 *
 */
void 
ShowRCASResults( RCASParameters *InRCAS )
{
    double min, max;
    double t1, t2;
    int display_times = DISPLAY_TIMING;
    double in_min, in_max;
    int minrow, mincol, maxrow, maxcol;
    int submat_minrow, submat_mincol, submat_maxrow, submat_maxcol;
    int num_rows, num_cols;
    char *key;
    char outbuff[500], debugmsg[500];
    int zc;

t1 = mytime();
    TRACE("ShowRCASResults called\n");
    /* calculate zoomcount */
    zc = InRCAS - RCAS;
    /* get max, min, input max ,etc. values */
    GetMaxMinEtc(InRCAS, &min, &max, &in_min, &in_max, 
		 &minrow, &mincol, &maxrow, &maxcol,
		 &submat_minrow, &submat_mincol, &submat_maxrow, 
		 &submat_maxcol, &num_rows, &num_cols, &key);

    sprintf(debugmsg,"ShowRCASResults: calculated zoomcount = %d\n",zc);
    DEBUG1(debugmsg);
    sprintf(debugmsg,"msg_label = %d\n",InRCAS->wids.msg_label);
    DEBUG2(debugmsg);


/* fprintf(stderr, "ShowRCASResults: Using "); 
 * if (InRCAS->sstrat == SClinear2)
 * {
 *    fprintf(stderr,"linear ");
 * }
 * else if (InRCAS->sstrat == SCabslinear)
 * {
 *   fprintf(stderr,"abs linear ");
 * }
 * else 
 * {
 *   if (InRCAS->log10 != 0)
 *	fprintf(stderr,"log 10 ");
 *   else
 *	fprintf(stderr, "log");
 * }
 */
/* 
 * fprintf(stderr," scaling. InRCAS->input_min = %f, InRCAS->input_absmin = %e\n",
 *	InRCAS->input_min,InRCAS->input_absmin);
 * fprintf(stderr,"calling SetColorMinMax: in_min = %e at (%d,%d)\n", in_min,minrow,mincol);
 */
    /* set the color bar straight */
    SetColorMinMax(min,max,
		   in_min, in_max,
		   minrow, mincol,
		   maxrow, maxcol,
		   zc);

    DEBUG2("ShowRCASResults: SetColorMinMax called\n");
    if (InRCAS->input->type == HB)
    {
	sprintf(outbuff, "Key = %s. Submatrix is from (%d, %d) to (%d, %d). Sparse Matrix is %d x %d with %d non-zeros.",
		key,
		submat_minrow,
		submat_mincol,
		submat_maxrow,
		submat_maxcol,
		num_rows,
		num_cols,
		InRCAS->input->hb_head.NumRowIndex);
    }
    else /* it's a DenseMatrix */
    {
	sprintf(outbuff, "Dense matrix is %d by %d.  Submatrix is from (%d, %d) to (%d, %d)",
		num_rows,
		num_cols,
		submat_minrow,
		submat_mincol,
		submat_maxrow,
		submat_maxcol);
    }
    sprintf(debugmsg,"outbuff = %s, zc = %d\n",outbuff,zc);
    DEBUG1(debugmsg);
    sprintf(debugmsg,"msg_label = %d\n",InRCAS->wids.msg_label);
    DEBUG2(debugmsg);

    SetMsgWinString(outbuff, zc);  
    DEBUG2("RCAS: outbuff written\n");

    /* the gc below is just any old gc, so no need to make 7 a const */
    CopyPixToCanvas( InRCAS->display, InRCAS->pix, InRCAS->window,
		     InRCAS->screen_height, InRCAS->screen_width, 
		     InRCAS->gcs[7] );

     XFlush(InRCAS->display);
    TRACE("ShowRCASResults done\n");
t2 = mytime();
if (display_times)
    printf("Display time = %e sec.\n",t2-t1);
}

void SetupFormAttachments( int zoomcount )
{
    Widget row_col, menu, msg, frame;

    TRACE("SetupFormAttachments: entered\n");
    row_col = RCAS[zoomcount].wids.row_col;
    menu = RCAS[zoomcount].wids.menu;
    msg = RCAS[zoomcount].wids.msg_win;
    frame = RCAS[zoomcount].wids.frame;

    DEBUG2("SetupFormAttachments: entered\n");
    XtVaSetValues(row_col,
		  XmNtopAttachment, XmATTACH_WIDGET,
		  XmNtopWidget, menu,
		  XmNbottomAttachment, XmATTACH_WIDGET,
		  XmNbottomWidget, msg,
		  XmNleftAttachment, XmATTACH_FORM,
		  XmNleftOffset, 1,
		  XmNrightAttachment, XmATTACH_NONE,
		  NULL);

    DEBUG2("SetupFormAttachments: row_col done -- trying frame\n");
    XtVaSetValues(frame,
		  XmNtopAttachment, XmATTACH_WIDGET,
		  XmNtopWidget, menu,
		  XmNtopOffset, 5,
		  XmNbottomAttachment, XmATTACH_WIDGET,
		  XmNbottomWidget, msg,
		  XmNbottomOffset, 5,
		  XmNleftAttachment, XmATTACH_WIDGET,
		  XmNleftWidget, row_col,
		  XmNrightAttachment, XmATTACH_FORM,
		  XmNrightOffset, 5,
		  NULL);

    DEBUG2("SetupFormAttachments: frame done -- trying msg\n");
    XtVaSetValues(msg,
		  XmNtopAttachment, XmATTACH_NONE,
		  XmNbottomAttachment, XmATTACH_FORM,
		  XmNbottomOffset, 1,
		  XmNleftAttachment, XmATTACH_FORM,
		  XmNleftOffset, 1,
		  XmNrightAttachment, XmATTACH_FORM,
		  XmNrightOffset, 1,
		  NULL);
    DEBUG2("SetupFormAttachments: done\n");
    TRACE("SetupFormAttachments: done\n");
}


void BuildChildrenWidgets(int zoomcount)
{
    Widget form_widget, row_col, toplvl;
    Widget cursor;
    char debugstr[100];

    if ((toplvl = RCAS[zoomcount].wids.toplevel) == (Widget) NULL)
    {
	fprintf(stderr,"BuildChildrenWidgets: toplevel uninitialized\n");
	return;
    }

    RCAS[zoomcount].wids.form = 
	form_widget = XtVaCreateManagedWidget(
	    "form_widget",		    /* instance name */
	    xmFormWidgetClass,              /* widget class */
	    toplvl,	                    /* parent */
	    NULL);			    /* terminate varargs list */
    
    /*
     * This RowColumn widget is for arranging the mouse position label
     * and the group of icons.
     */
    
    RCAS[zoomcount].wids.row_col = 
    row_col = XtVaCreateManagedWidget(
	"row_column1",		/* instance name */
	xmRowColumnWidgetClass,	/* widget class */
	form_widget,		/* parent */
	XmNallowResize, False,      /* hmmm... */
	NULL);			/* terminate varargs list */

    
    DEBUG2( "made it past form widget creation\n");
    /*
     * Make the menus
     */
    
    MakeMenus(form_widget, zoomcount); /* tjl */
    
    /* ok so far */
    /*
     * Make Label widget for holding the cursor position text.
     */

    MakeCursorPos(row_col, zoomcount);
    DEBUG2("made it past MakeCursorPos\n");


    /*
     * Make the canvas widget.  This is a DrawingArea widget that we
     * do the drawing in.
     */

    MakeCanvas(form_widget, zoomcount);

    DEBUG2("made it past MakeCanvas\n");

	/*
	 * Make the column of icons.  They are put in a RowColumn widget to
	 * maintain their shape.
	 */

    /* if (zoomcount == 0) */
	MakeIcons(row_col, zoomcount);
    DEBUG2("past MakeIcons\n");
    MakeColorBar(row_col, zoomcount);
    DEBUG2("past MakeColorBar\n");
    /* if (zoomcount == 0) */
	InitializeModes(zoomcount);
    DEBUG2("modes initted\n");

    /*
     * Make a ScrolledWindow widget to hold the message.
     */
    
    MakeMsgWin(form_widget, zoomcount);
    
    DEBUG2("after MakeMsgWin\n");
    SetMsgWinString("Colors have been re-allocated.", zoomcount); 
    DEBUG2("after SetMsgWinString\n");

	
    /*
     * Now we need to set all the resource values to specify how the children
     * of the form widget will be attached.
     * 
     * This is done in the SetupFormAttachments function.
     *
     */

    SetupFormAttachments(zoomcount);
    DEBUG2("form widget setup\n");

    /* save all the widgets */
}
