#include <hb.h>

/*
 * File: zoom.c
 *
 * References:
 *  
 *  Volume Six: Motif Programming Manual for OSF/Motif Version 1.1, 
 *  O'Reilly and Associates, Sebastopol, CA
 *
 * Includes: ZoomMat()
 *
 * Started: 3/22/94
 *
 * Modified: 
 *
 */

/*
 * Function: ZoomMat()
 *
 * Purpose: to display an expanded view of a matrix
 *
 * Parameters: z - a pointer to a ZoomParms structure
 *
 * Usage: zoomcount = ZoomMat( &z );
 *
 * Return Values: new zoomcount if OK otherwise -1
 * 
 * Calls: AllocRCAS  BuildChildrenWidgets SetXGlobals
 *        ReadCompressAndScaleMat DEBUG fprintf
 *	  ShowRCASResults
 *
 * Programmer: Tom Loos, Indiana Univ, LH310, +01 (812) 855-3609,
 *             tloos@cs.indiana.edu
 *
 * Started:   3/22/94
 *
 * Modified:  3/24/94 -- rough draft done
 *            ...
 *            4/19/94 replace XFillRectangle with CanvasClearWindow()
 *
 */
int ZoomMat( ZoomParms *z )
{
    int rv, oldzc;
    char debugmsg[100];
    char zname[20];
    static Arg Zoom_xargs[20];
    Cursor cursor;

    TRACE("ZoomMat called\n");

    /* get a new RCAS */
    rv = AllocRCAS( RCAS, MAXZOOM );
    if (rv < 0)
    {
	fprintf(stderr,"ZoomMat: couldn't get a free window\n");
	return(rv);
    }
    sprintf(debugmsg,"ZoomMat: rv = %d, zc = %d\n",rv,z->zoomcount);
/* fprintf(stderr,"%s",debugmsg);*/
    DEBUG2(debugmsg);

    oldzc = z->zoomcount;
    /* set the old window's cursor to the watch cursor */
    SetCursor(XC_watch, &cursor);
    /* INITIALIZE WITH PREVIOUS RCAS */

    RCAS[rv].input = RCAS[oldzc].input;
    RCAS[rv].cstrat = RCAS[oldzc].cstrat;
    RCAS[rv].sstrat = RCAS[oldzc].sstrat;
    RCAS[rv].conformal = RCAS[oldzc].conformal;

    RCAS[rv].startcolor = RCAS[oldzc].startcolor;
    RCAS[rv].numcolors = RCAS[oldzc].numcolors;
    RCAS[rv].force_black = RCAS[oldzc].force_black;
    RCAS[rv].what_is_black = RCAS[oldzc].what_is_black;
    RCAS[rv].display = RCAS[oldzc].display;
    RCAS[rv].gcs = RCAS[oldzc].gcs;

#ifdef HANDLE_PTNS
    /* assign ptn counts */
    RCAS[rv].NumRowPtns = RCAS[oldzc].NumRowPtns;
    RCAS[rv].NumColPtns = RCAS[oldzc].NumColPtns;

    RCAS[rv].UsedRowPtns = RCAS[oldzc].NumRowPtns;
    RCAS[rv].UsedColPtns = RCAS[oldzc].NumColPtns;

    /* and assign ptn pointers */
    RCAS[rv].RowPtn = RCAS[oldzc].RowPtn;
    RCAS[rv].ColPtn = RCAS[oldzc].ColPtn;
    RCAS[rv].vectfile =  RCAS[oldzc].vectfile;

    /* if we have ptns, and there is no local/new row ptn storage, get some. */
    if ((RCAS[rv].NumRowPtns > 1) && (RCAS[rv].NewRowPtn == (int*) NULL))
    {
	/* if malloc fails, set num ptns to 0 */
	if ((RCAS[rv].NewRowPtn = MALLOC(MAX_PTNS, sizeof(int))) == 
	    (int*) NULL)
	{
	    fprintf(stderr,"Failed to get row ptn storage for window %d\n",
		    rv);
	    RCAS[rv].NumRowPtns = 0;
	}
	
    }
    /* if we have ptns, and there is no local/new col ptn storage, get some. */
    if ((RCAS[rv].NumColPtns > 1) && (RCAS[rv].NewColPtn == (int*) NULL))
    {
	/* if malloc fails, set num ptns to 0 */
	if ((RCAS[rv].NewColPtn = MALLOC(MAX_PTNS, sizeof(int))) == 
	    (int*) NULL)
	{
	    fprintf(stderr,"Failed to get col ptn storage for window %d\n",
		    rv);
	    RCAS[rv].NumColPtns = 0;
	}
	
    }
#endif

    /* commented out on 9/1/94 */
    /*RCAS[rv].input = RCAS[oldzc].input; */

    /* START THIS RCAS's DATA */
    RCAS[rv].mapped = 0;
    RCAS[rv].action = RCAScompress;
    RCAS[rv].input_startrow = z->input_startrow;
    RCAS[rv].input_startcol = z->input_startcol;
    RCAS[rv].input_endrow = z->input_endrow;
    RCAS[rv].input_endcol = z->input_endcol;

    /* 10/5/94 -- handle zero row/col problems */

    if (RCAS[rv].input_startrow == RCAS[rv].input_endrow)
    {
	if (RCAS[rv].input_startrow != 0)
	    RCAS[rv].input_startrow--;
	else
	    RCAS[rv].input_endrow++;
    }
    if (RCAS[rv].input_startcol == RCAS[rv].input_endcol)
    {
	if (RCAS[rv].input_startcol != 0)
	    RCAS[rv].input_startcol--;
	else
	    RCAS[rv].input_endcol++;
    }
/*
 *   fprintf(stderr,"input start = %d,%d, input end = %d, %d\n",
 *	    RCAS[rv].input_startrow, RCAS[rv].input_startcol,
 *	    RCAS[rv].input_endrow, RCAS[rv].input_endcol);
 */

    RCAS[rv].screen_width = z->screen_width;
    RCAS[rv].screen_height = z->screen_height;

    sprintf(debugmsg,"ZoomMat: building kid wids\n");
    DEBUG2(debugmsg);
    sprintf(zname,"zoom_win%d",rv);
    XtSetArg(Zoom_xargs[0], XmNcolormap, Default_cmap); 
    XtSetArg(Zoom_xargs[1], XmNmappedWhenManaged, False);   
    XtSetArg(Zoom_xargs[2], XmNvisual, canvas_visual); 
    XtSetArg(Zoom_xargs[3], XmNdepth, canvas_depth); 
    /* create a TOP LEVEL shell */
    RCAS[rv].wids.toplevel = XtAppCreateShell( "zoom window",
					       zname,
					       topLevelShellWidgetClass,
					       canvas_display,
					       Zoom_xargs,
					       4);

    /* make the kid wids and hook them up */
    BuildChildrenWidgets(rv);

    /* create the widget, but don't show it yet */
    XtRealizeWidget(RCAS[rv].wids.toplevel);  

    /* post realization cleanup */
    sprintf(debugmsg,"ZoomMat: setting globals\n");
    DEBUG2(debugmsg);
    SetXGlobals(rv);
    RCAS[rv].window = XtWindow(RCAS[rv].wids.canvas);
    FillColorBar(RCAS[rv].wids.color_bar,rv);

    /* create pixmap */
    RCAS[rv].pixalloc = 1;
    RCAS[rv].pix = XCreatePixmap(RCAS[rv].display,
				 RCAS[rv].window, /* try this */
				 RCAS[rv].screen_width,
				 RCAS[rv].screen_height,
				 canvas_depth);
    CanvasClearWindow(rv);
    /*
     * XFillRectangle(RCAS[rv].display, RCAS[rv].pix, RCAS[rv].gcs[BG_INDEX],
     *		   0,0,RCAS[rv].screen_width,RCAS[rv].screen_height);
     */
    
    sprintf(debugmsg,"ZoomMat: rv = %d, window = %d\n",RCAS[rv].window);
/* fprintf(stderr,"%s",debugmsg);*/
    DEBUG2(debugmsg);
    sprintf(debugmsg,"ZoomMat: width = %d, height = %d\n",RCAS[rv].screen_width,
	    RCAS[rv].screen_height);
    DEBUG2(debugmsg);
    RCAS[rv].comp = (DenseMatrix *) MALLOC(1,sizeof(DenseMatrix));
    if (ReadCompressAndScaleMat( &RCAS[rv] ) == OK)
    {
	/* DEBUG1("RCAS OK: copy pix\n"); */
	/* commit to new window! */
	sprintf(debugmsg,"ZoomMat: RCAS OK! Showing results...\n");
	DEBUG2(debugmsg);
	ShowRCASResults( &RCAS[rv] );
    }

    /* XtManageChild(RCAS[rv].wids.toplevel);  // not good for top level */

    /* show your work for full credit ... */
    XtMapWidget(RCAS[rv].wids.toplevel);  
    /* and reset the old window's cursor */
    ResetCursor(cursor);
    RCAS[rv].mapped = 1;
    TRACE("ZoomMat: done\n");
    return(rv);
}






